/****************************************************************/
/*                                                              */
/*  defines.v                                                   */
/*                                                              */
/*  Author:   Tianmo Lei (tm2008@yahoo.com)                     */
/*  Version:  V0.1                                              */
/*  Date:     Feb 15, 2002                                      */
/*  Additional information is avaliable in the Readme.txt       */
/*                                                              */
/****************************************************************/
/*                                                              */
/* Copyright (C) 2002 Tianmo Lei                                */
/*                                                              */
/* This source file may be used and distributed without         */
/* restriction provided that this copyright statement is not    */
/* removed from the file and that any derivative work contains  */
/* the original copyright notice and the associated disclaimer. */
/*                                                              */
/* This source file is free software; you can redistribute it   */
/* and/or modify it under the terms of the GNU Lesser General   */
/* Public License as published by the Free Software Foundation; */
/* either version 2.1 of the License, or (at your option) any   */
/* later version.                                               */
/* This source is distributed in the hope that it will be       */
/* useful, but WITHOUT ANY WARRANTY; without even the implied   */
/* warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      */
/* PURPOSE. See the GNU Lesser General Public License for more  */
/* details.                                                     */
/*                                                              */
/****************************************************************/

/********************** Athmatic Operation **********************/ 
`define ALU_NOP		4'b0000 	// no operation
`define ALU_ADD		4'b0001 	// add
`define ALU_SUB		4'b0010		// sub
`define ALU_MUL		4'b0011		// mul
`define ALU_DIV		4'b0100		// div
`define ALU_DA 		4'b0101		// decimal adjust
`define ALU_NOT		4'b0110		// not
`define ALU_AND		4'b0111		// and
`define ALU_OR 		4'b1000		// or
`define ALU_XOR		4'b1001		// xor
`define ALU_RLC		4'b1010		// rotate left with Cy
`define ALU_RL 		4'b1011		// rotate left
`define ALU_RRC		4'b1100		// rotate right with Cy
`define ALU_RR 		4'b1101		// rotate right
`define ALU_SWP		4'b1110		// A half byte exchange
`define ALU_XCH		4'b1111		// xch, xchd
   
/******************** Byte address SFR **************************/
`define SFR_B   	8'hf0		// B
`define SFR_ACC 	8'he0		// A
`define SFR_PSW 	8'hd0		// PSW
`define SFR_IP 		8'hb8		// interrupt priority
`define SFR_P3  	8'hb0		// P3
`define SFR_IE 		8'ha8		// interrupt enable
`define SFR_P2  	8'ha0		// P2
`define SFR_SBUF 	8'h99		// serial port buffer
`define SFR_SCON 	8'h98		// serial port control
`define SFR_P1  	8'h90		// P1
`define SFR_TH1 	8'h8d		// timer1 high byte
`define SFR_TH0 	8'h8c		// timer0 high byte
`define SFR_TL1 	8'h8b		// timer1 low byte
`define SFR_TL0 	8'h8a		// timer0 low byte
`define SFR_TMOD 	8'h89		// timer mode
`define SFR_TCON 	8'h88		// timer control
`define SFR_PCON 	8'h87		// timer power control
`define SFR_DPTR_HI	8'h83		// dptr low byte
`define SFR_DPTR_LO	8'h82		// dptr high byte
`define SFR_SP 		8'h81		// stack pointer
`define SFR_P0  	8'h80		// P0

/******************** Bit address SFR ***************************/
`define SFR_B_B 	5'b11110	// B
`define SFR_B_ACC 	5'b11100	// A
`define SFR_B_PSW 	5'b11010	// PSW
`define SFR_B_IP 	5'b10111	// interrupt priority
`define SFR_B_P3 	5'b10110	// P3
`define SFR_B_IE 	5'b10101	// interrupt enable
`define SFR_B_P2 	5'b10100	// P2
`define SFR_B_SCON 	5'b10011	// serial port control
`define SFR_B_P1 	5'b10010	// P1
`define SFR_B_TCON 	5'b10001	// timer control
`define SFR_B_P0 	5'b10000	// P0

/******************** 8051 Instruction **************************/

/* 5 bit instruction [4:0]                   bytes cycles       */
`define ACALL		5'b1_0001	// 2 2 absolute call
`define AJMP		5'b0_0001	// 2 2 absolute jump
   
/* 5 bit instruction [7:3]           bytes cycles   	 	*/
`define INC_R		5'b0000_1 //xxx  1 1 Rn++
`define DEC_R		5'b0001_1 //xxx  1 1 Rn--
`define ADD_R		5'b0010_1 //xxx  1 1 A=A+Rx
`define ADDC_R 		5'b0011_1 //xxx  1 1 A=A+Rx+Cy
`define ORL_R		5'b0100_1 //xxx  1 1 A=A|Rn
`define ANL_R		5'b0101_1 //xxx  1 1 A=A&Rx
`define XRL_R		5'b0110_1 //xxx  1 1 A=A^Rn
`define MOV_RC 		5'b0111_1 //xxx  2 2 Rn=constant
`define MOV_DR 		5'b1000_1 //xxx  2 2 (direct)=Rn
`define SUBB_R 		5'b1001_1 //xxx  1 1 A=A-Rn-Cy
`define MOV_RD 		5'b1010_1 //xxx  2 2 Rn=(direct)
`define CJNE_R 		5'b1011_1 //xxx  3 2 jump if Rx!=constant
`define XCH_R		5'b1100_1 //xxx  1 1 A exchange with Rn
`define DJNZ_R 		5'b1101_1 //xxx  2 2 Rn--, jump if Rn!=0
`define MOV_R		5'b1110_1 //xxx  1 1 A=Rn
`define MOV_RA 		5'b1111_1 //xxx  1 1 Rn=A
  
/* 7 bit instruction [7:1]           bytes cycles               */
`define INC_I		7'b0000_011 //x  1 1 @Ri++
`define DEC_I		7'b0001_011 //x  1 1 @Ri--
`define ADD_I		7'b0010_011 //x  1 1 A=A+@Ri
`define ADDC_I		7'b0011_011 //x  1 1 A=A+@Ri+Cy
`define ORL_I		7'b0100_011 //x  1 1 A=A|@Ri
`define ANL_I		7'b0101_011 //x  1 1 A=A&@Ri
`define XRL_I		7'b0110_011 //x  1 1 A=A^@Ri
`define MOV_IC		7'b0111_011 //x  2 1 @Ri=constant
`define MOV_DI		7'b1000_011 //x  2 2 (direct)=@Ri
`define SUBB_I		7'b1001_011 //x  1 1 A=A-@Ri-Cy
`define MOV_ID		7'b1010_011 //x  2 2 @Ri=(direct)
`define CJNE_I		7'b1011_011 //x  3 2 jump if @Ri!=constant
`define XCH_I		7'b1100_011 //x  1 1 A exchange with @Ri
`define XCHD		7'b1101_011 //x  1 1 A low half byte exchange with @Ri
`define MOV_I		7'b1110_011 //x  1 1 A=@Ri
`define MOV_IA		7'b1111_011 //x  1 1 @Ri=A
`define MOVX_I		7'b1110_001 //x  1 2 A=exter_ram @Ri
`define MOVX_IA 	7'b1111_001 //x  1 2 exter_ram @Ri=A
   
/* 8 bit instruction [7:0]           bytes cycles       	*/
`define NOP		8'b0000_0000 //  1 1 no operation
`define LJMP		8'b0000_0010 //  3 2 long jump
`define RR		8'b0000_0011 //  1 1 rotate right
`define INC_A		8'b0000_0100 //  1 1 A++
`define INC_D		8'b0000_0101 //  2 1 (direct)++
`define JBC		8'b0001_0000 //  3 2 jump if bit==1, clear bit
`define LCALL		8'b0001_0010 //  3 2 long call
`define RRC		8'b0001_0011 //  1 1 rotate right with Cy
`define DEC_A		8'b0001_0100 //  1 1 A--
`define DEC_D		8'b0001_0101 //  2 1 (direct)--
`define JB		8'b0010_0000 //  3 2 jump if bit==1
`define RET		8'b0010_0010 //  1 2 return from subrutine
`define RL		8'b0010_0011 //  1 1 rotate left
`define ADD_C		8'b0010_0100 //  2 1 A=A+constant
`define ADD_D		8'b0010_0101 //  2 1 A=A+(direct)
`define JNB		8'b0011_0000 //  3 2 jump if bit==0
`define RETI		8'b0011_0010 //  1 2 return from interrupt
`define RLC		8'b0011_0011 //  1 1 rotate left with Cy
`define ADDC_C		8'b0011_0100 //  2 1 A=A+constant+Cy
`define ADDC_D		8'b0011_0101 //  2 1 A=A+(direct)+Cy
`define JC		8'b0100_0000 //  2 2 jump if Cy==1
`define ORL_DA		8'b0100_0010 //  2 1 (direct)=(direct)|A
`define ORL_DC		8'b0100_0011 //  3 2 (direct)=(direct)|constant
`define ORL_C		8'b0100_0100 //  2 1 A=A|constant
`define ORL_D		8'b0100_0101 //  2 1 A=A|(direct)
`define JNC		8'b0101_0000 //  2 2 jump if Cy==0
`define ANL_DA		8'b0101_0010 //  2 1 (direct)=(direct)&A
`define ANL_DC		8'b0101_0011 //  3 2 (direct)=(direct)&constant
`define ANL_C		8'b0101_0100 //  2 1 A=A&constant
`define ANL_D		8'b0101_0101 //  2 1 A=A&(direct)
`define JZ		8'b0110_0000 //  2 2 jump if A==0
`define XRL_DA		8'b0110_0010 //  2 1 (direct)=(direct)^A
`define XRL_DC		8'b0110_0011 //  3 2 (direct)=(direct)^constant
`define XRL_C		8'b0110_0100 //  2 1 A=A^constant
`define XRL_D		8'b0110_0101 //  2 1 A=A^(direct)
`define JNZ		8'b0111_0000 //  2 2 jump if A==1
`define ORL_B		8'b0111_0010 //  2 1 Cy=Cy|bit
`define JMP		8'b0111_0011 //  1 2 jump indirect
`define MOV_C		8'b0111_0100 //  2 1 A=constant
`define MOV_DC		8'b0111_0101 //  3 2 (direct)=constant
`define SJMP		8'b1000_0000 //  2 2 short jump
`define ANL_B		8'b1000_0010 //  2 1 Cy=Cy&bit
`define MOVC_PC 	8'b1000_0011 //  1 2 A=(pc+A)
`define DIV		8'b1000_0100 //  1 4 A/B
`define MOV_DD		8'b1000_0101 //  3 2 (direct)=(direct)
`define MOV_DPC 	8'b1001_0000 //  3 2 dptr=constant
`define MOV_BC		8'b1001_0010 //  2 1 bit=Cy
`define MOVC_DP 	8'b1001_0011 //  1 2 A=(dptr+A)
`define SUBB_C		8'b1001_0100 //  2 1 A=A-constant-Cy
`define SUBB_D		8'b1001_0101 //  2 1 A=A-(direct)-Cy
`define ORL_NB		8'b1010_0000 //  2 1 Cy=Cy|~bit
`define MOV_CB		8'b1010_0010 //  2 1 Cy=bit
`define INC_DP		8'b1010_0011 //  1 1 dptr++
`define MUL		8'b1010_0100 //  1 4 A*B
`define ANL_NB		8'b1011_0000 //  2 1 Cy=Cy&~bit
`define CPL_B		8'b1011_0010 //  2 1 ~bit
`define CPL_C		8'b1011_0011 //  1 1 ~Cy
`define CJNE_C		8'b1011_0100 //  3 2 jump if A!=constant
`define CJNE_D		8'b1011_0101 //  3 2 jump if A!=(direct)
`define PUSH		8'b1100_0000 //  2 2 push to stack
`define CLR_B		8'b1100_0010 //  2 1 bit=0
`define CLR_C		8'b1100_0011 //  1 1 Cy=0
`define SWAP		8'b1100_0100 //  1 1 A(0-3) swap with A(4-7)
`define XCH_D		8'b1100_0101 //  2 1 A exchange with (direct)
`define POP		8'b1101_0000 //  2 2 pop from stack
`define SETB_B		8'b1101_0010 //  2 1 bit=1
`define SETB_C		8'b1101_0011 //  1 1 Cy=1
`define DA		8'b1101_0100 //  1 1 decimal adjust
`define DJNZ_D		8'b1101_0101 //  3 2 (direct)--, jump if (direct)!=0
`define MOVX_AP 	8'b1110_0000 //  1 2 A=(dptr)
`define CLR_A		8'b1110_0100 //  1 1 A=0
`define MOV_D		8'b1110_0101 //  2 1 A=(direct)
`define MOVX_PA 	8'b1111_0000 //  1 2 (dptr)=A
`define CPL_A		8'b1111_0100 //  1 1 ~A
`define MOV_DA		8'b1111_0101 //  2 1 (direct)=A
    
/************************* Miscellaneus *************************/
/* instruction bytes                                            */
`define ONEbyte		2'b01
`define TWObyte		2'b10
`define THREEbyte	2'b11

/* instruction cycles                                           */
`define ONEcyc		2'b01
`define TWOcyc		2'b10
`define THREEcyc	2'b11
`define FOURcyc		2'b00

/* instruction cycle                                            */
`define FIRST		4'b0001
`define SECOND		4'b0010
`define THIRD		4'b0100
`define FORTH		4'b1000

/* tmp1, tmp2 input select                                      */
`define ACC		2'b10
`define B		2'b10 
`define D8BUS		2'b01
`define A8BUS		2'b11
`define ZERO8		2'b00
`define ONE8		2'b01

/* tmpb input select   		   				*/
`define ZERO		3'b000
`define ONE		3'b001 
`define D1_N		3'b011
`define D1BUS		3'b010
`define CY		3'b100
   
/* 16 bit address bus (a16bus) input select 			*/
`define A16BUF		2'b01
`define PC		2'b00
`define DP		2'b10
  
