/****************************************************************/
/*                                                              */
/*  L8051.v                                                     */
/*                                                              */
/*  Author:   Tianmo Lei (tm2008@yahoo.com)                     */
/*  Version:  V0.1                                              */
/*  Date:     Feb 15, 2002                                      */
/*  Additional information is avaliable in the Readme.txt       */
/*                                                              */
/****************************************************************/
/*                                                              */
/* Copyright (C) 2002 Tianmo Lei                                */
/*                                                              */
/* This source file may be used and distributed without         */
/* restriction provided that this copyright statement is not    */
/* removed from the file and that any derivative work contains  */
/* the original copyright notice and the associated disclaimer. */
/*                                                              */
/* This source file is free software; you can redistribute it   */
/* and/or modify it under the terms of the GNU Lesser General   */
/* Public License as published by the Free Software Foundation; */
/* either version 2.1 of the License, or (at your option) any   */
/* later version.                                               */
/* This source is distributed in the hope that it will be       */
/* useful, but WITHOUT ANY WARRANTY; without even the implied   */
/* warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      */
/* PURPOSE. See the GNU Lesser General Public License for more  */
/* details.                                                     */
/*                                                              */
/****************************************************************/
 
`timescale 100ps / 100ps
`include "defines.v"

module l8051 (  .\1 (p1[0]),  .\2 (p1[1]),  .\3 (p1[2]),  .\4 (p1[3]),
     		.\5 (p1[4]),  .\6 (p1[5]),  .\7 (p1[6]),  .\8 (p1[7]),
     		.\9 (rst),    .\10 (p3[0]), .\11 (p3[1]), .\12 (p3[2]),
     		.\13 (p3[3]), .\14 (p3[4]), .\15 (p3[5]), .\16 (p3[6]),
     		.\17 (p3[7]), .\18 (xtal2), .\19 (xtal1), .\20 (gnd),
     		.\21 (p2[0]), .\22 (p2[1]), .\23 (p2[2]), .\24 (p2[3]),
     		.\25 (p2[4]), .\26 (p2[5]), .\27 (p2[6]), .\28 (p2[7]),
     		.\29 (psen),  .\30 (ale),   .\31 (ea),	  .\32 (p0[7]),
     		.\33 (p0[6]), .\34 (p0[5]), .\35 (p0[4]), .\36 (p0[3]),
     		.\37 (p0[2]), .\38 (p0[1]), .\39 (p0[0]), .\40 (vcc) );
     
inout	[7:0] p0, p1, p2, p3;	
input	rst;			
input	ea,xtal1;	
output  ale,psen,xtal2;	
input	vcc,gnd;		 
     					// BUS: 
wire	[15:0]	a16bus;				// address
wire	[7:0]	d8bus,				// data
 		a8bus,				// ram_address
 		instruction;			// instruction
wire		d1bus;				// one bit bus 
					// CLK:
wire		clk,reset;			// clk
reg		ph1, ph2;			// two phase
reg	[5:0]	state;				// six state
wire		state1 = state[0],		//
 		state2 = state[1],		//
 		state3 = state[2],		//
 		state4 = state[3],		//
 		state5 = state[4],		//
   		state6 = state[5];		//
reg	[3:0] 	cycle;				// instruction CYCLES
wire		cycle1 = cycle[0],		//
         	cycle2 = cycle[1],		//
         	cycle3 = cycle[2],		//
         	cycle4 = cycle[3];		//
wire		lastcyc;			//
reg 		Cy,Ac,f0,rs1,rs0,Ov,f1,Pa;	// PSW
     
reg  		reti_flg,ip_flg,ie_flg,int_i,int_r,			// interupt ctrl
   		x_wr,x_rd,ext_rw,					// ext access ctrl
       		rd_ram,rd_ram_b,wr_ram,wr_ram_b,rd_time,wr_time,p0_to_a8,// ram ctrl
       		sp_add_1,sp_to_a8,sp_sub_1,				// sp ctrl
 		pc_h_to_d8,pc_l_to_d8,d8_to_pc_h,d8_to_pc_l,		// pc ctrl
 		a16b_l_to_a8,a16b_l_to_d8,d8_to_a16b_h,d8_to_a16b_l,	// tmp buf ctrl
 		a16b_h_to_d8,
		ir1_to_d8,ir2_to_d8,ir3_to_d8,ir2_to_a8,ir3_to_a8,      // ir ctrl
		rn_to_a8,ri_to_a8, 
 		dp_l_to_a8,dp_h_to_a8,					// dptr ctrl
		wr_acc,rd_acc,clr_acc,not_acc,wr_b,			// acc,b ctrl
       		ld_tmp1,ld_tmp2_l,ld_tmp2_h,ld_tmpb,			// alu src ctrl
 		des_h_to_d8,des_l_to_d8,desCy_to_d1,			// alu des ctrl
		saveAc,saveOv,saveCy,Z_reg,not_cy,clr_cy,set_cy,        // alu lfg ctrl
		Cy_to_d1,sign,Ac_to_d8,
		txdbuf,							// s_port
   		p2_sel,dirc_flg,indirc_flg;				// port output sel
reg	[3:0]	alu_op,				// alu operation
		new_int,crt_int;		// interrupt name
reg	[2:0]	srcb;				// alu src sel 
reg	[1:0]	src1,src2,			// alu src sel 
		a16bus_src,			// address bus sel 
		i_byte,				// instruction bytes 
	 	cycno;				// instruction cyc number 
wire		p0_sel,p3_0_sel,		// port output sel
 		pa,				// parity flg
 		desZ,accZ,			// zero flg
 		p0_to_d8;			// p0 to data bus enable
 
integer 	here,there; 
 

/******************* CLK ********************/
 
reg	ale_i, psen_i;
wire	ea_o;
   
assign xtal2 = ~clk;
assign clk = ~xtal1;
assign reset = rst;
assign psen = psen_i;
assign ale =ale_i;
assign ea_o = ea;

initial
begin
  state= 6'b0; ph1 = 0; ph2 = 0;
  ale_i = 0; psen_i = 0;
end
  
always @(negedge clk or posedge reset)
  if(reset==1)
  begin ph1 = 0; ph2 = 0;
  end
  else
  begin
    ph2=0; #1 ph1=1;
    @(negedge clk) begin ph1=0; #1 ph2=1; end
  end
   
always @(negedge ph2 or posedge reset)
  if(reset) state= 6'b0;
  else
  begin
    state = 6'b001000;
    @(negedge ph2) state = 6'b010000;
    @(negedge ph2) state = 6'b100000;
    @(negedge ph2) state = 6'b000001;
    @(negedge ph2) state = 6'b000010;
    @(negedge ph2) state = 6'b000100;
  end
   
always @(negedge ph1 or posedge reset)
  if(reset) ale_i = 0; 
  else
  begin
    if(state1 & ext_rw) ale_i = 1;
    if(state4) ale_i = 1;
    if(state2|state5) ale_i = 0;
  end
   
always @(negedge ph1 or posedge reset)
  if(reset) psen_i = 0;
  else
    if(state1|state4)
    begin
      psen_i = 1;
      @(negedge ph1)
      @(negedge ph2)
      if(ext_rw) psen_i = 0;
    end
 
/******************* CLK ***********************/
 
/******************* PORT **********************/
     
reg	[7:0] pp0,pp1,pp2,pp3;                  // port register
reg	txdbuf0;
wire	p0_oe,p1_oe,p2_oe,p3_oe;
wire	rxd,int0,int1,t0,t1;
wire	[7:0] p0_i = p0;			// port input
wire	[7:0] p1_i = p1;
wire	[7:0] p2_i = p2;
wire	[7:0] p3_i = p3;
wire	[7:0] p0 = ~p0_oe ? (p0_sel ? a16bus[7:0] : pp0) : 'bz;      
wire	[7:0] p1 = ~p1_oe ?  pp1 : 'bz;  
wire	[7:0] p2 = ~p2_oe ? (p2_sel ? a16bus[15:8] : pp2) : 'bz;
wire	[7:0] p3;

assign p3[7] = ~p3_oe ? x_rd : 'bz;
assign p3[6] = ~p3_oe ? x_wr : 'bz;
assign p3[5:2] = 'bz;
assign p3[1] = ~p3_oe ? txdbuf : 'bz;
assign p3[0] = ~(p3_0_sel | p3_oe) ? txdbuf0 : 'bz;
   
assign p0_sel  = state5 | (state2&ext_rw);
     
assign rxd  = p3_i[0];
assign int0 = p3_i[2]; 
assign int1 = p3_i[3]; 
assign t0   = p3_i[4]; 
assign t1   = p3_i[5]; 
     
always @(posedge clk or posedge reset)          // write port register
  if(reset)
  begin
    pp0 = 8'b0; pp1 = 8'b0; pp2 = 8'b0; pp3 = 8'b0;
  end
  else
  begin
    if(wr_ram & wr_time & dirc_flg)
      case(a8bus)
      `SFR_P0: pp0 = d8bus;
      `SFR_P1: pp1 = d8bus;
      `SFR_P2: pp2 = d8bus;
      `SFR_P3: pp3 = d8bus;
      endcase
    else if(wr_ram_b & wr_time & dirc_flg)         
      case(a8bus[7:3])
      `SFR_B_P0: pp0[a8bus[2:0]] = d1bus;
      `SFR_B_P1: pp1[a8bus[2:0]] = d1bus;
      `SFR_B_P2: pp2[a8bus[2:0]] = d1bus;
      `SFR_B_P3: pp3[a8bus[2:0]] = d1bus;
    endcase
  end
						   // port read condition 
wire rd_p0,rd_p1,rd_p2,rd_p3,rd_p0_b,rd_p1_b,rd_p2_b,rd_p3_b;
assign rd_p0 = rd_ram & (a8bus==`SFR_P0) & dirc_flg;
assign rd_p1 = rd_ram & (a8bus==`SFR_P1) & dirc_flg;
assign rd_p2 = rd_ram & (a8bus==`SFR_P2) & dirc_flg;
assign rd_p3 = rd_ram & (a8bus==`SFR_P3) & dirc_flg;
assign rd_p0_b = rd_ram_b & (a8bus[7:3]==`SFR_B_P0) & dirc_flg;
assign rd_p1_b = rd_ram_b & (a8bus[7:3]==`SFR_B_P1) & dirc_flg;
assign rd_p2_b = rd_ram_b & (a8bus[7:3]==`SFR_B_P2) & dirc_flg;
assign rd_p3_b = rd_ram_b & (a8bus[7:3]==`SFR_B_P3) & dirc_flg;

reg r_m_w_instr;                                  // read modify write instruction
always @(instruction or reset)
if(reset) r_m_w_instr=0;
else
  case(instruction)
  `ANL_DA,`ANL_DC, `ORL_DA,`ORL_DC, `XRL_DA,`XRL_DC,
  `CPL_B,`CLR_B,`SETB_B,`MOV_BC,`INC_D,`DEC_D,`JBC,`DJNZ_D:
    r_m_w_instr=1;
  default:
    r_m_w_instr=0;
  endcase

wire rd_p0p,rd_p1p,rd_p2p,rd_p3p,rd_p0_bp,rd_p1_bp,rd_p2_bp,rd_p3_bp;
assign rd_p0p = rd_p0 & ~r_m_w_instr | p0_to_d8;   // read port input
assign rd_p1p = rd_p1 & ~r_m_w_instr;
assign rd_p2p = rd_p2 & ~r_m_w_instr;
assign rd_p3p = rd_p3 & ~r_m_w_instr;
assign rd_p0_bp = rd_p0_b & ~r_m_w_instr | p0_to_d8; 
assign rd_p1_bp = rd_p1_b & ~r_m_w_instr;
assign rd_p2_bp = rd_p2_b & ~r_m_w_instr;
assign rd_p3_bp = rd_p3_b & ~r_m_w_instr;
assign d8bus = rd_p0p ? p0_i : 8'bz;
assign d8bus = rd_p1p ? p1_i : 8'bz;
assign d8bus = rd_p2p ? p2_i : 8'bz;
assign d8bus = rd_p3p ? p3_i : 8'bz;
assign d1bus = rd_p0_bp ? p0_i[a8bus[2:0]] : 1'bz;
assign d1bus = rd_p1_bp ? p1_i[a8bus[2:0]] : 1'bz;
assign d1bus = rd_p2_bp ? p2_i[a8bus[2:0]] : 1'bz;
assign d1bus = rd_p3_bp ? p3_i[a8bus[2:0]] : 1'bz;
   
wire rd_p0r,rd_p1r,rd_p2r,rd_p3r,rd_p0_br,rd_p1_br,rd_p2_br,rd_p3_br;
assign rd_p0r = rd_p0 & r_m_w_instr;               // read port registers
assign rd_p1r = rd_p1 & r_m_w_instr;
assign rd_p2r = rd_p2 & r_m_w_instr;
assign rd_p3r = rd_p3 & r_m_w_instr;
assign rd_p0_br = rd_p0_b & r_m_w_instr;               
assign rd_p1_br = rd_p1_b & r_m_w_instr;
assign rd_p2_br = rd_p2_b & r_m_w_instr;
assign rd_p3_br = rd_p3_b & r_m_w_instr;
assign d8bus = rd_p0r ? pp0 : 8'bz;
assign d8bus = rd_p1r ? pp1 : 8'bz;
assign d8bus = rd_p2r ? pp2 : 8'bz;
assign d8bus = rd_p3r ? pp3 : 8'bz;
assign d1bus = rd_p0_br ? pp0[a8bus[2:0]] : 1'bz;
assign d1bus = rd_p1_br ? pp1[a8bus[2:0]] : 1'bz;
assign d1bus = rd_p2_br ? pp2[a8bus[2:0]] : 1'bz;
assign d1bus = rd_p3_br ? pp3[a8bus[2:0]] : 1'bz;
   
assign p0_oe = rd_p0 | rd_p0_b | p0_to_d8 ? 1 : 0; // port output enable
assign p1_oe = rd_p1 | rd_p1_b ? 1 : 0;
assign p2_oe = rd_p2 | rd_p2_b ? 1 : 0;
assign p3_oe = rd_p3 | rd_p3_b ? 1 : 0;
     
/******************* PORT **********************/
 
/******************* RAM **********************/
 
reg [7:0] ram1[0:31];
reg [127:0] ram2;
reg [7:0] ram3[48:127];
reg [7:0] ram4[128:255];
     
always @(posedge clk)
  if(wr_ram & wr_time)
  begin
     if(a8bus[7] & indirc_flg) ram4[a8bus] = #1 d8bus;
     else if(~a8bus[7])
     begin
       if(a8bus>47) ram3[a8bus] = #1 d8bus;
       else if(a8bus<32) ram1[a8bus] = #1 d8bus;
       else ram2[a8bus] = #1 d8bus;
     end
  end
  else if(wr_ram_b & wr_time & ~a8bus[7])
    ram2[a8bus] = #1 d1bus;
     
assign d8bus = rd_ram & a8bus[7] & indirc_flg ? ram4[a8bus] : 8'bz;
assign d8bus = rd_ram & ~a8bus[7] & a8bus>47 ? ram3[a8bus] : 8'bz;
assign d8bus = rd_ram & a8bus<32 ? ram1[a8bus] : 8'bz;
assign d1bus = rd_ram_b & ~a8bus[7] ? ram2[a8bus] : 1'bz;

/******************* RAM **********************/

/******************* ACC_TMP1 **********************/
 
reg tmpb,tmpCy;
reg [7:0] acc,tmp1;
    
// ACC
   
always @(posedge clk or posedge reset)
  if(reset)
  begin
     acc = #1 8'b0; clr_acc=0; not_acc=0;
  end
  else if(wr_time & wr_ram & (a8bus==`SFR_ACC) & dirc_flg) acc = #1 d8bus;
  else if(wr_time & wr_ram_b & (a8bus[7:3]==`SFR_B_ACC) & dirc_flg) acc[a8bus[2:0]] = #1 d1bus;
  else if(wr_acc) acc = #1 d8bus;
  else if(clr_acc) acc = 0;
  else if(not_acc) acc = ~acc;
  
assign d8bus = (rd_ram & (a8bus==`SFR_ACC) & dirc_flg) | rd_acc ? acc : 8'bz;
assign d1bus = rd_ram_b & (a8bus[7:3]==`SFR_B_ACC) & dirc_flg ? acc[a8bus[2:0]] : 1'bz;
assign pa = ^acc;
assign accZ=|acc;

// TMP1
   
always @(posedge clk or posedge reset)
  if(reset)
    tmp1 = #1 8'b0;
  else
    if(ld_tmp1)
      case(src1)
        `ACC: tmp1 = #1 acc;
        `ZERO8: tmp1 = #1 2'h0;
        `D8BUS: tmp1 = #1 d8bus;
        `ONE8: tmp1 = #1 2'h1;
      endcase
   
// TMPB
   
always @(posedge clk or posedge reset)
  if(reset)
  begin
    tmpb = #1 0; tmpCy=0;
  end
  else
    if(ld_tmpb)
    begin
      case(srcb)
      `D1_N: tmpb = #1 ~d1bus;
      `D1BUS: tmpb = #1 d1bus;
      `ZERO: tmpb = #1 0;
      `ONE: tmpb = #1 1;
      `CY: tmpb = #1 Cy;
      endcase
      tmpCy = Cy;
    end
   
/******************* ACC_TMP1 **********************/
 

/******************* B_TMP2  ***********************/  
  
reg [7:0] b_reg;
reg [15:0] tmp2;

always @(posedge clk or posedge reset)
  if(reset)
  begin b_reg = #1 8'b0; wr_b=0; end
  else if(wr_ram & wr_time & (a8bus==`SFR_B) & dirc_flg) b_reg = #1 d8bus;
  else if(wr_ram_b & wr_time & (a8bus[7:3]==`SFR_B_B) & dirc_flg) 
    b_reg[a8bus[2:0]] = #1 d1bus;
  else if(wr_b)
    b_reg = #1 d8bus;
  
assign d8bus = rd_ram & (a8bus==`SFR_B) & dirc_flg ? b_reg : 8'bz;
assign d1bus = rd_ram_b & (a8bus[7:3]==`SFR_B_B) & dirc_flg ? b_reg[a8bus[2:0]] : 1'bz;
   
// TMP2
   
always @(posedge clk or posedge reset)
  if(reset) tmp2 = #1 8'b0;
  else if(ld_tmp2_l)
    case(src2)
    `B: tmp2[7:0] = #1 b_reg;
    `ZERO: tmp2[7:0] = #1 8'b0;
    `D8BUS: tmp2[7:0] = #1 d8bus;
    `A8BUS: tmp2[7:0] = #1 a8bus;
    endcase
  else if(ld_tmp2_h)
    case(src2)
    `B: tmp2[15:8] = #1 b_reg;
    `ZERO: tmp2[15:8] = #1 8'b0;
    `D8BUS: tmp2[15:8] = #1 d8bus;
    `A8BUS: tmp2[15:8] = #1 a8bus;
    endcase
   
/******************* B_TMP2  ***********************/  
 
/******************* ALU_PSW ***********************/  
     
reg [15:0] des;
reg desCy,desAc,desOv,Ov1;
reg [8:0] da1;
     
// ALU
 
assign desZ = |des[7:0];
 
always @(alu_op or tmp1 or tmp2 or tmpb) //posedge clk)
begin
  case(alu_op)
  `ALU_ADD:
  begin
    {desAc,des[3:0]} = tmp1[3:0]+tmp2[3:0] + tmpb;
    {Ov1,des[6:4]} = tmp1[6:4] + tmp2[6:4] + desAc;
    {desCy,des[7]} = tmp1[7] + tmp2[7] + Ov1;
    desOv = desCy ^ Ov1;
    if(sign==1) 
      if(tmp1[7]==0) des[15:8] = tmp2[15:8]+desCy;
      else des[15:8]=8'b11111111+tmp2[15:8]+desCy;
    else des[15:8] = tmp2[15:8]+desCy;
  end
  `ALU_SUB:
  begin
    {desAc,des[3:0]} = tmp1[3:0] - tmp2[3:0] - tmpb;
    {Ov1,des[6:4]} = tmp1[6:4] - tmp2[6:4] - desAc;
    {desCy,des[7]} = tmp1[7] - tmp2[7] - Ov1;
    desOv = desCy ^ Ov1;
  end
  `ALU_MUL:
  begin
    des = tmp1 * tmp2[7:0];
    if(des[15:8] != 8'b0) desOv=1;
    else desOv=0;
    desCy=0;
  end
  `ALU_DIV:
  begin
    if(tmp2[7:0]==0) desOv=1;
    else if(tmp2[7:0]>tmp1) begin des[7:0]=0; des[15:8]=tmp1; desOv=0; end
    else begin des[7:0]=tmp1/tmp2[7:0]; des[15:8]=tmp1%tmp2[7:0]; desOv=0; end
    desCy=0;
  end
  `ALU_DA:
  begin
    if(tmp1[3:0]>9 | tmp2[0]==1)
      {desAc,des[3:0]}=tmp1[3:0]+6;
    else begin des[3:0]=tmp1[3:0]; desAc=tmp2[0]; end
    if(tmp1[7:4]>9) 
      {desCy,des[7:4]}=tmp1[7:4]+6+desAc;
    else if(tmpb==1)
      begin des[7:4]=tmp1[7:4]+6+desAc; desCy=tmpb; end
    else
      {desCy,des[7:4]}=tmp1[7:4]+desAc;
  end
  `ALU_NOT:
  begin des[7:0] = ~tmp1; desCy = ~tmpb;
  end
  `ALU_AND:
  begin
    des[7:0] = tmp1 & tmp2[7:0];
    desCy=tmpCy&tmpb;
  end
  `ALU_XOR:
  begin
    des[7:0] = tmp1 ^ tmp2[7:0];
    desCy=tmpCy & ~tmpb;
  end
  `ALU_OR:
  begin
    des[7:0] = tmp1 | tmp2[7:0];
    desCy= tmpCy | tmpb;
  end
  `ALU_RL:
  begin des[7:0] = {tmp1[6:0], tmp1[7]};
  end
  `ALU_RLC:
  begin
    des[7:0] = {tmp1[6:0], Cy};
    des[15:8]= {tmp1[3:0], tmp1[7:4]};
    desCy=tmp1[7];
  end
  `ALU_RR:
  begin des[7:0] = {tmp1[0], tmp1[7:1]};
  end
  `ALU_RRC:
  begin des[7:0] = {Cy, tmp1[7:1]}; desCy=tmp1[0];
  end
  `ALU_XCH:
    if(tmpb)
    begin des[7:0]=tmp2[7:0];des[15:8]=tmp1; end
    else begin des[7:0]={tmp1[7:4],tmp2[3:0]};des[15:8]={tmp2[7:4],tmp1[3:0]}; end
  `ALU_SWP:
  begin des={tmp1[3:0],tmp1[7:4]};
  end
  `ALU_NOP:
  begin des[15:8]=tmp2[7:0]; des[7:0]=tmp1; desCy=tmpb;
  end
  endcase
end
     
// PSW
     
wire [7:0] psw;

always @(posedge clk or posedge reset)
  if(reset)
  begin
    {Cy,Ac,f0,rs1,rs0,Ov,f1,Pa} = #1 8'b0; clr_cy=0; not_cy=0;
  end
  else if(wr_ram & wr_time & (a8bus==`SFR_PSW) & dirc_flg)
    {Cy,Ac,f0,rs1,rs0,Ov,f1,Pa} = #1 d8bus;
  else if(wr_ram_b & wr_time & (a8bus[7:3]==`SFR_B_PSW) & dirc_flg)
    case(a8bus[2:0])
    3'b000: Pa = #1 d1bus;
    3'b001: f1 = #1 d1bus;
    3'b010: Ov = #1 d1bus;
    3'b011: rs0 = #1 d1bus;
    3'b100: rs1 = #1 d1bus;
    3'b101: f0 = #1 d1bus;
    3'b110: Ac = #1 d1bus;
    3'b111: Cy = #1 d1bus;
    endcase
    
always @(posedge clk)
  if(saveCy) Cy=desCy;
  else if(not_cy) Cy=~Cy;
  else if(clr_cy) Cy=0;
  else if(set_cy) Cy=1;

always @(posedge clk)
  if(saveAc) Ac=desAc;
always @(posedge clk)
  if(saveOv) Ov=desOv;
always @(posedge clk)
  if(lastcyc & state6 & ph2) Pa=pa;
     
assign d8bus = des_h_to_d8 ? des[15:8] : 8'bz;
assign d8bus = des_l_to_d8 ? des[7:0] : 8'bz;
assign a8bus[4:3] = (rn_to_a8|ri_to_a8) ? {rs1,rs0} : 2'bz;

assign d8bus = rd_ram & (a8bus==`SFR_PSW) & dirc_flg ? psw : 8'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_PSW,3'b000}) & dirc_flg ? Pa : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_PSW,3'b001}) & dirc_flg ? f1 : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_PSW,3'b010}) & dirc_flg ? Ov : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_PSW,3'b011}) & dirc_flg ? rs0 : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_PSW,3'b100}) & dirc_flg ? rs1 : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_PSW,3'b101}) & dirc_flg ? f0 : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_PSW,3'b110}) & dirc_flg ? Ac : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_PSW,3'b111}) & dirc_flg ? Cy : 1'bz;
assign d1bus = Cy_to_d1 ? Cy : 1'bz;
assign d1bus = desCy_to_d1 ? desCy : 1'bz;
     
/******************* ALU_PSW ***********************/  
   
/******************* IR_IE_IP_TCON *****************/  
     
reg	[7:0]	ir1,ir2,ir3;                // IR
reg		EA,Es,ET2,ES,ET1,EX1,ET0,EX0,     // IE
		Ps0,Ps1,PT2,PS,PT1,PX1,PT0,PX0,   // IP
		TF1,TR1,TF0,TR0,IE1,IT1,IE0,IT0;  // TCON
wire	[7:0]	ie={EA,Es,ET2,ES,ET1,EX1,ET0,EX0},     // IE
		ip={Ps0,Ps1,PT2,PS,PT1,PX1,PT0,PX0},     // IP
		tcon={TF1,TR1,TF0,TR0,IE1,IT1,IE0,IT0};  // TCON
  
assign instruction=ir1;
  
// INSTRUCTION REGISTER
 
always @(posedge clk or posedge reset or posedge int_r)
  if(reset)
  begin  ir1 = #1 8'b0; ir2 = #1 8'b0; ir3 = #1 8'b0;
  end
  else if(int_r)                         // state1
// INTERRUPT PROCESS
  begin
    @(posedge state1) ir1=`NOP;  
    @(posedge state6) @(posedge ph2)            // state6
    case(crt_int)
    4'b0000,
    4'b1000: IE0=0;
    4'b0001,
    4'b1001: TF0=0; 
    4'b0010,
    4'b1010: IE1=0;
    4'b0011,
    4'b1011: TF1=0;
    4'b0100,
    4'b1100: ;
    endcase
  end
// INTERRUPT PROCESS
  else if(state1 & ph1 & cycle1)
    ir1 = #1 d8bus;
  else if(state4 & ph1 & cycle1)
    ir2 = #1 d8bus;
  else if(state1 & ph1 & cycle2)
    ir3 = #1 d8bus;
      
assign d8bus = ir1_to_d8 ? ir1 : 8'bz;
assign d8bus = ir2_to_d8 ? ir2 : 8'bz;
assign d8bus = ir3_to_d8 ? ir3 : 8'bz;
assign a8bus = ir2_to_a8 ? ir2 : 8'bz;
assign a8bus = ir3_to_a8 ? ir3 : 8'bz;
assign a8bus[7:5] = (rn_to_a8|ri_to_a8) ? 3'b0 : 3'bz;
assign a8bus[2:0] = rn_to_a8 ? ir1[2:0] : 3'bz;
assign a8bus[2:0] = ri_to_a8 ? {2'b0,ir1[0]} : 3'bz;
    
// IE,IP,TCON
     
always @(posedge clk or posedge reset)
  if(reset)
    {EA,Es,ET2,ES,ET1,EX1,ET0,EX0} = #1 8'b0;
  else if(wr_ram & wr_time & (a8bus==`SFR_IE) & dirc_flg)
    {EA,Es,ET2,ES,ET1,EX1,ET0,EX0} = #1 d8bus;
  else if(wr_ram_b & wr_time & (a8bus[7:3]==`SFR_B_IE) & dirc_flg)
    case(a8bus[2:0])
    3'b111: EA = #1 d1bus;
    3'b110: Es = #1 d1bus;
    3'b101: ET2 = #1 d1bus;
    3'b100: ES = #1 d1bus;
    3'b011: ET1 = #1 d1bus;
    3'b010: EX1 = #1 d1bus;
    3'b001: ET0 = #1 d1bus;
    3'b000: EX0 = #1 d1bus;
    endcase
     
assign d8bus = rd_ram & (a8bus==`SFR_IE) & dirc_flg  ? ie : 8'bz;
assign d1bus = rd_ram_b & (a8bus[7:3]=={`SFR_B_IE,3'b111}) & dirc_flg  ? EA : 1'bz;
assign d1bus = rd_ram_b & (a8bus[7:3]=={`SFR_B_IE,3'b110}) & dirc_flg  ? Es : 1'bz;
assign d1bus = rd_ram_b & (a8bus[7:3]=={`SFR_B_IE,3'b101}) & dirc_flg  ? ET2 : 1'bz;
assign d1bus = rd_ram_b & (a8bus[7:3]=={`SFR_B_IE,3'b100}) & dirc_flg  ? ES : 1'bz;
assign d1bus = rd_ram_b & (a8bus[7:3]=={`SFR_B_IE,3'b011}) & dirc_flg  ? ET1 : 1'bz;
assign d1bus = rd_ram_b & (a8bus[7:3]=={`SFR_B_IE,3'b010}) & dirc_flg  ? EX1 : 1'bz;
assign d1bus = rd_ram_b & (a8bus[7:3]=={`SFR_B_IE,3'b001}) & dirc_flg  ? ET0 : 1'bz;
assign d1bus = rd_ram_b & (a8bus[7:3]=={`SFR_B_IE,3'b000}) & dirc_flg  ? EX0 : 1'bz;
     
always @(posedge clk or posedge reset)
  if(reset)
    {Ps0,Ps1,PT2,PS,PT1,PX1,PT0,PX0} = #1 8'b0;
  else if(wr_ram & wr_time & (a8bus==`SFR_IP) & dirc_flg)
    {Ps0,Ps1,PT2,PS,PT1,PX1,PT0,PX0} = #1 d8bus;
  else if(wr_ram_b & wr_time & (a8bus[7:3]==`SFR_B_IP) & dirc_flg)
    case(a8bus[2:0])
    3'b111: Ps0 = #1 d1bus;
    3'b110: Ps1 = #1 d1bus;
    3'b101: PT2 = #1 d1bus;
    3'b100: PS = #1 d1bus;
    3'b011: PT1 = #1 d1bus;
    3'b010: PX1 = #1 d1bus;
    3'b001: PT0 = #1 d1bus;
    3'b000: PX0 = #1 d1bus;
    endcase
     
assign d8bus = rd_ram & (a8bus==`SFR_IP) & dirc_flg ? ip : 8'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_IP,3'b111}) & dirc_flg ? Ps0 : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_IP,3'b110}) & dirc_flg ? Ps1 : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_IP,3'b101}) & dirc_flg ? PT2 : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_IP,3'b100}) & dirc_flg ? PS : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_IP,3'b011}) & dirc_flg ? PT1 : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_IP,3'b010}) & dirc_flg ? PX1 : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_IP,3'b001}) & dirc_flg ? PT0 : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_IP,3'b000}) & dirc_flg ? PX0 : 1'bz;
     
always @(posedge clk or posedge reset)
  if(reset)
    {TF1,TR1,TF0,TR0,IE1,IT1,IE0,IT0} = #1 8'b0;
  else if(wr_ram & wr_time & (a8bus==`SFR_TCON) & dirc_flg)
    {TF1,TR1,TF0,TR0,IE1,IT1,IE0,IT0} = #1 d8bus;
  else if(wr_ram_b & wr_time & (a8bus[7:3]==`SFR_B_TCON) & dirc_flg)
    case(a8bus[2:0])
    3'b111: TF1 = #1 d1bus;
    3'b110: TR1 = #1 d1bus;
    3'b101: TF0 = #1 d1bus;
    3'b100: TR0 = #1 d1bus;
    3'b011: IE1 = #1 d1bus;
    3'b010: IT1 = #1 d1bus;
    3'b001: IE0 = #1 d1bus;
    3'b000: IT0 = #1 d1bus;
    endcase
     
assign d8bus = rd_ram & (a8bus==`SFR_TCON) & dirc_flg ? tcon : 8'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_TCON,3'b111}) & dirc_flg ? TF1 : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_TCON,3'b110}) & dirc_flg ? TR1 : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_TCON,3'b101}) & dirc_flg ? TF0 : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_TCON,3'b100}) & dirc_flg ? TR0 : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_TCON,3'b011}) & dirc_flg ? IE1 : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_TCON,3'b010}) & dirc_flg ? IT1 : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_TCON,3'b001}) & dirc_flg ? IE0 : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_TCON,3'b000}) & dirc_flg ? IT0 : 1'bz;
     
/******************* IR_IE_IP_TCON *****************/
 
/******************* PC_ABUF_DPTR ******************/  
   
reg [15:0] dptr;
reg [15:0] pc,a16buf;
     
always @(posedge clk or posedge reset or posedge int_r)
  if(reset) pc = 16'b0;
  else if(int_r & state1) 
// INTERRUPT PROCESS
  begin
    @(posedge state4) 
    @(posedge ph2)
    case(crt_int)
    4'b0000,
    4'b1000: pc ='h0003;
    4'b0001,
    4'b1001: pc ='h0006;//b; 
    4'b0010,
    4'b1010: pc ='h0009;//13;
    4'b0011,
    4'b1011: pc ='h000c;//1b;
    4'b0100,
    4'b1100: pc ='h000f;//23;
    endcase
  end
// INTERRUPT PROCESS
  else if(cycle1&state1&ph2)
    pc = pc + 1;
  else if(cycle1&state4&ph2 & (i_byte==2 | i_byte==3))
    pc = pc + 1;
  else if(cycle2&state1&ph2 & (i_byte==3))
    pc = pc + 1;
  else if(d8_to_pc_l)
    pc[7:0] = #1 d8bus;
  else if(d8_to_pc_h)
    if(instruction[3:0] == 4'b0001)
      pc[10:8] = #1 d8bus[7:5];
    else
      pc[15:8] = #1 d8bus;
     
assign d8bus = pc_l_to_d8 ? pc[7:0] : 8'bz;
assign d8bus = pc_h_to_d8 ? pc[15:8] : 8'bz;
    
assign a16bus  = a16bus_src==`PC ?  pc
 		 : a16bus_src==`DP ? dptr
  	        : a16bus_src==`A16BUF ? a16buf : 16'bz;
  
// ADDRESS BUF
  
always @(posedge clk or posedge reset)
  if (reset) a16buf = 16'b0;
  else if(d8_to_a16b_h)
    a16buf[15:8] = d8bus;
  else if(d8_to_a16b_l)
    a16buf[7:0] = d8bus;
     
assign a8bus = a16b_l_to_a8 ? a16buf[7:0] : 8'bz;
assign d8bus = a16b_l_to_d8 ? a16buf[7:0] : 8'bz;
assign d8bus = a16b_h_to_d8 ? a16buf[15:8] : 8'bz;
     
   
// DATA POINTER
   
always @(posedge clk or posedge reset)
  if(reset) dptr = 16'b0;
  else if(wr_ram & wr_time & dirc_flg)
  begin
    if(a8bus==`SFR_DPTR_HI)
      dptr[15:8] = #1 d8bus;
    else if(a8bus==`SFR_DPTR_LO)
      dptr[7:0] = #1 d8bus;
  end
 
assign d8bus = rd_ram & (a8bus==`SFR_DPTR_HI) & dirc_flg ? dptr[15:8] : 8'bz;
assign d8bus = rd_ram & (a8bus==`SFR_DPTR_LO) & dirc_flg ? dptr[7:0] : 8'bz;
   
/******************* PC_ABUF_DPTR ******************/
 
/******************* SP ****************************/  
  
reg [7:0] sp;
 
always @(posedge clk or posedge reset)
  if(reset)
    sp = #1 7;
  else if(sp_sub_1)
    sp = sp - 1;
  else if(sp_add_1)
    sp = sp + 1;
  else if(wr_ram & (a8bus==`SFR_SP) & wr_time & dirc_flg)
    sp = #1 d8bus;
   
assign d8bus = rd_ram & (a8bus==`SFR_SP) & dirc_flg ? sp : 8'bz;
assign a8bus = sp_to_a8 ? sp : 8'bz;
  
/******************* SP ****************************/  
 
/**************** COUNTER **************************/  

reg [7:0] th0,tl0,th1,tl1;
reg t1_flg,t0_flg,cntr1_Ov;
reg GATE1,CT1,M11,M10,GATE0,CT0,M01,M00;   
wire [7:0] tmod={GATE1,CT1,M11,M10,GATE0,CT0,M01,M00};  

always @(posedge clk or posedge reset)
  if(reset) 
  begin 
    th0=0; th1=0; tl0=0; tl1=0;  cntr1_Ov=0;
    {GATE1,CT1,M11,M10,GATE0,CT0,M01,M00}=8'b0;
  end
  else if(wr_ram & wr_time & dirc_flg)
    case(a8bus)
    `SFR_TMOD: {GATE1,CT1,M11,M10,GATE0,CT0,M01,M00} = #1 d8bus;
    `SFR_TH0: th0 = #1 d8bus;
    `SFR_TH1: begin th1 = #1 d8bus; cntr1_Ov=0; end
    `SFR_TL0: tl0 = #1 d8bus;
    `SFR_TL1: begin tl1 = #1 d8bus; cntr1_Ov=0; end
    endcase
assign d8bus = rd_ram & (a8bus==`SFR_TMOD) & dirc_flg ? tmod : 8'bz;
assign d8bus = rd_ram & (a8bus==`SFR_TH0) & dirc_flg ? th0 : 8'bz;
assign d8bus = rd_ram & (a8bus==`SFR_TH1) & dirc_flg ? th1 : 8'bz;
assign d8bus = rd_ram & (a8bus==`SFR_TL0) & dirc_flg ? tl0 : 8'bz;
assign d8bus = rd_ram & (a8bus==`SFR_TL1) & dirc_flg ? tl1 : 8'bz;

always @(negedge state4 or posedge reset)
if(reset) t0_flg=0; 
else 
begin
  case({M01,M00})
  2'b00: 
  begin
    tl0[7:5]=3'b0;
    if((GATE0==0 | (GATE0==1 & int0==1)) & TR0==1) 
    begin
      if(CT0==0) 
      begin 
        @(posedge ph2) if(tl0[4:0]=='h1f) th0=th0+1;
	 tl0[4:0]=tl0[4:0]+1;
      end
      else 
      begin
        @(posedge ph2) 
        if(t0==1) t0_flg=1;
	 else if(t0_flg==1) 
        begin
	   @(posedge state3)  if(tl0[4:0]==5'h1f) th0=th0+1;
	   tl0[4:0]=tl0[4:0]+1;
	   t0_flg=0;
        end 
      end
      if({th0,tl0[4:0]}==13'b0) TF0=1;
    end
  end
  2'b01: 
  begin
    if((GATE0==0 | (GATE0==1 & int0==1)) & TR0==1) 
    begin
      if(CT0==0) 
      begin 
        @(posedge ph2) if(tl0==8'hff) th0=th0+1;
	 tl0=tl0+1;
      end
      else 
      begin
        @(posedge ph2) 
	 if(t0==1) t0_flg=1;
	 else if(t0_flg==1) 
        begin
	   @(posedge state3) if(tl0==8'hff) th0=th0+1;
 	   tl0=tl0+1;
	   t0_flg=0;
        end 
      end
      if({th0,tl0}==16'b0) TF0=1;
    end
  end
  2'b10: 
  begin
    if((GATE0==0 | (GATE0==1 & int0==1)) & TR0==1) 
    begin
      if(CT0==0) @(posedge ph2) tl0=tl0+1; 
      else 
      begin
        @(posedge ph2) 
        if(t0==1) t0_flg=1;
	 else if(t0_flg==1) 
        begin
	   @(posedge state3) tl0=tl0+1;
	   t0_flg=0;
        end 
      end
      if(tl0==0) begin tl0=th0; TF0=1; end
    end
  end
  2'b11: 
  begin
    if((GATE0==0 | (GATE0==1 & int0==1)) & TR0==1) 
    begin
      if(CT0==0) @(posedge ph2) tl0=tl0+1; 
      else 
      begin
        @(posedge ph2) 
	 if(t0==1) t0_flg=1;
	 else if(t0_flg==1) 
        begin 
	   @(posedge state3) tl0=tl0+1; 
	   t0_flg=0; 
	 end 
      end
      if(tl0==0) TF0=1;
    end
    if(TR1==1) 
    begin
      @(posedge ph2) th0=th0+1;
      if(th0==0) TF1=1;
    end
  end
  endcase
end

always @(negedge state4 or posedge reset)
if(reset) begin t1_flg=0; cntr1_Ov=0; end
else if({M01,M00}!=2'b11)  
begin
  case({M11,M10})
  2'b00: 
  begin
    tl1[7:5]=3'b0;
    if((GATE1==0 | (GATE1==1 & int1==1)) & TR1==1) 
    begin
      if(CT1==0) 
	 begin 
          @(posedge ph2) if(tl1[4:0]==5'h1f) th1=th1+1;
	   tl1[4:0]=tl1[4:0]+1;
        end
      else 
      begin
        @(posedge ph2) 
	 if(t1==1) t1_flg=1;
	 else if(t1_flg==1) 
	 begin 
          @(posedge state3) if(tl1[4:0]==5'h1f) th1=th1+1;
	   tl1[4:0]=tl1[4:0]+1;
	   t1_flg=0;
        end 
      end
      if({th1,tl1[4:0]}==13'b0) begin TF1=1; cntr1_Ov=1; end 
      else cntr1_Ov=0;
    end
  end
  2'b01: 
  begin
    if((GATE1==0 | (GATE1==1 & int1==1)) & TR1==1) 
    begin
      if(CT1==0) 
      begin 
        @(posedge ph2) if(tl1==8'hff) th1=th1+1;
        tl1=tl1+1;
        end
      else 
      begin
        @(posedge ph2) 
	 if(t1==1) t1_flg=1;
	 else if(t1_flg==1) 
	 begin 
          @(posedge state3) if(tl1==8'hff) th1=th1+1;
	   tl1=tl1+1;
	   t1_flg=0;
        end 
      end
      if({th1,tl1}==16'b0) begin TF1=1; cntr1_Ov=1; end
      else cntr1_Ov=0;
    end
  end
  2'b10: 
  begin
    if((GATE1==0 | (GATE1==1 & int1==1)) & TR1==1) 
    begin
      if(CT1==0)
        @(posedge ph2) if(tl1==0) tl1=th1; 
	 else tl1=tl1+1;
      else 
      begin
        @(posedge ph2) 
	 if(t1==1) t1_flg=1;
	 else if(t1_flg==1) 
          begin
	   @(posedge state3) tl1=tl1+1;
	   t1_flg=0;
	 end
      end
      if(tl1==0) begin TF1=1; cntr1_Ov=1; end 
      else cntr1_Ov=0;
    end
  end
  2'b11: ;
  endcase
end
else
begin
  case({M11,M10})
  2'b00: 
  begin
    tl1[7:5]=3'b0;
    if(GATE1==0 | (GATE1==1 & int1==1))  
    begin
      if(CT1==0) 
      begin 
        @(posedge ph2) if(tl1[4:0]==5'h1f) th1=th1+1;
        tl1[4:0]=tl1[4:0]+1;
      end
      else 
      begin
        @(posedge ph2) 
        if(t1==1) t1_flg=1;
	 else if(t1_flg==1) 
	 begin 
          @(posedge state3) if(tl1[4:0]==5'h1f) th1=th1+1;
	   tl1[4:0]=tl1[4:0]+1;
	   t1_flg=0;
        end 
      end
      if({th1,tl1[4:0]}==13'b0) cntr1_Ov=1; 
      else cntr1_Ov=0;
    end
  end
  2'b01: 
  begin
    if(GATE1==0 | (GATE1==1 & int1==1)) 
    begin
      if(CT1==0) 
      begin 
        @(posedge ph2) if(tl1==8'hff) th1=th1+1;
	 tl1=tl1+1;
      end
      else 
      begin
        @(posedge ph2) 
	 if(t1==1) t1_flg=1;
	 else if(t1_flg==1) 
	 begin 
          @(posedge state3) if(tl1==8'hff) th1=th1+1;
	   tl1=tl1+1;
	   t1_flg=0;
        end 
      end
      if({th1,tl1}==16'b0) cntr1_Ov=1; 
      else cntr1_Ov=0;
    end
  end
  2'b10: 
  begin
    if(GATE1==0 | (GATE1==1 & int1==1)) 
    begin
      if(CT1==0) @(posedge ph2) tl1=tl1+1; 
      else 
      begin
        @(posedge ph2) 
        if(t1==1) t1_flg=1;
	 else if(t1_flg==1) 
        begin
	   @(posedge state3) tl1=tl1+1;
	   t1_flg=0;
	 end
      end
      if(tl1==0) begin tl1=th1; cntr1_Ov=1; end 
      else cntr1_Ov=0;
    end
  end
  2'b11: ;
  endcase
end

/**************** COUNTER **************************/  

/*************** SERIAL PORT ***********************/  

reg [8:0] rcvbuf;
reg [7:0] sbufr,sbuft;
reg [3:0] tcntr,rcntr;
reg SM0,SM1,SM2,REN,TB8,RB8,TI,RI;  
reg SMOD,PW0,PW1,PW2,GF1,GF0,PD,IDL;
wire [7:0] scon={SM0,SM1,SM2,REN,TB8,RB8,TI,RI},  
           pcon={SMOD,PW0,PW1,PW2,GF1,GF0,PD,IDL};
reg sclk,sm1,sm2,sm3;
wire tcntr0=~(|tcntr), rcntr0=~(|rcntr),
     rcntr7=(~rcntr[3])&(&rcntr[2:0]);
reg tx_flg;

always @(posedge clk or posedge reset)
  if(reset) 
  begin 
    {SM0,SM1,SM2,REN,TB8,RB8,TI,RI}=8'b0;  
    {SMOD,PW0,PW1,PW2,GF1,GF0,PD,IDL}=8'b0;
    tx_flg=0;
  end
  else if(wr_ram & wr_time & dirc_flg)
    case(a8bus)
    `SFR_SCON: {SM0,SM1,SM2,REN,TB8,RB8,TI,RI}=#1 d8bus;  
    `SFR_PCON: {SMOD,PW0,PW1,PW2,GF1,GF0,PD,IDL}=#1 d8bus;
    `SFR_SBUF: begin sbuft= #1 d8bus; tx_flg=1; end
    endcase
  else if(wr_ram_b & wr_time & (a8bus[7:3]==`SFR_B_SCON) & dirc_flg)
    case(a8bus[2:0])
    3'b000: RI = #1 d1bus;
    3'b001: TI = #1 d1bus;
    3'b010: RB8 = #1 d1bus;
    3'b011: TB8 = #1 d1bus;
    3'b100: REN = #1 d1bus;
    3'b101: SM2 = #1 d1bus;
    3'b110: SM1 = #1 d1bus;
    3'b111: SM0 = #1 d1bus;
    endcase

assign d8bus = rd_ram & (a8bus==`SFR_SBUF) & dirc_flg ? sbufr : 8'bz;
assign d8bus = rd_ram & (a8bus==`SFR_SCON) & dirc_flg ? scon : 8'bz;
assign d8bus = rd_ram & (a8bus==`SFR_PCON) & dirc_flg ? pcon : 8'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_SCON,3'b000}) & dirc_flg ? RI : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_SCON,3'b001}) & dirc_flg ? TI : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_SCON,3'b010}) & dirc_flg ? RB8 : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_SCON,3'b011}) & dirc_flg ? TB8 : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_SCON,3'b100}) & dirc_flg ? REN : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_SCON,3'b101}) & dirc_flg ? SM2 : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_SCON,3'b110}) & dirc_flg ? SM1 : 1'bz;
assign d1bus = rd_ram_b & (a8bus=={`SFR_B_SCON,3'b111}) & dirc_flg ? SM0 : 1'bz;

always @(posedge tx_flg)
  if(SM0 | SM1)
  begin
    @(posedge tcntr0) txdbuf=0;
    @(posedge tcntr0) txdbuf=sbuft[0]; sbuft[6:0]=sbuft[7:1];
    @(posedge tcntr0) txdbuf=sbuft[0]; sbuft[6:0]=sbuft[7:1];
    @(posedge tcntr0) txdbuf=sbuft[0]; sbuft[6:0]=sbuft[7:1];
    @(posedge tcntr0) txdbuf=sbuft[0]; sbuft[6:0]=sbuft[7:1];
    @(posedge tcntr0) txdbuf=sbuft[0]; sbuft[6:0]=sbuft[7:1];
    @(posedge tcntr0) txdbuf=sbuft[0]; sbuft[6:0]=sbuft[7:1];
    @(posedge tcntr0) txdbuf=sbuft[0]; sbuft[6:0]=sbuft[7:1];
    @(posedge tcntr0) txdbuf=sbuft[0]; sbuft[6:0]=sbuft[7:1];
    if({SM0,SM1}==2'b10) @(posedge tcntr0) txdbuf=TB8;
    @(posedge tcntr0) txdbuf=1;
    @(posedge tcntr0) TI=1; 
    tx_flg=0;
  end
  else
  begin
    txdbuf=1;
    @(posedge state1)
    repeat(8)
    begin
      @(posedge state1) txdbuf0=sbuft[0]; sbuft[6:0]=sbuft[7:1];
      @(posedge state3) txdbuf=0;
      @(posedge state6) txdbuf=1;
    end
    @(posedge state1) TI=1;
  end

always @(posedge ph2 or posedge reset)
if(reset) sclk=0;
else if({SM0,SM1}==2'b10)
  if(SMOD==0) @(posedge ph2) sclk=~sclk;
  else sclk=~sclk;

always @(posedge cntr1_Ov or posedge reset)
if(reset) sclk=0;
else
case({SM0,SM1})
2'b01,2'b11:
  if(SMOD==0) @(posedge cntr1_Ov) sclk=~sclk;
  else sclk=~sclk;
endcase  

always @(posedge sclk or posedge reset)
if(reset) begin rcntr=0; tcntr=0; end
else
case({SM0,SM1})
  2'b01,2'b10,2'b11: begin rcntr=rcntr+1; tcntr=tcntr+1; end
endcase  

integer j;
always @(posedge sclk or posedge reset)
if(reset) begin sm1=0; sm2=0; sm3=0; rcvbuf=0; sbufr=0; end
else
begin
  sm1=rxd;
  if(~sm1&REN&(SM0|SM1))
  begin
    rcntr=0; 
    @(posedge rcntr7) sm1=rxd; @(posedge sclk) sm2=rxd; @(posedge sclk) sm3=rxd;
    if((~sm1 & ~sm2)|(~sm1 & ~sm3)|(~sm2 & ~sm3))
    begin
      for(j=0;j<9;j=j+1) 
      begin
        @(posedge rcntr7) sm1=rxd; @(posedge sclk) sm2=rxd; @(posedge sclk) sm3=rxd;
        @(posedge sclk) rcvbuf[j]= (sm1==sm2)?sm1:(sm1==sm3)?sm1:sm2;
      end
      if(~RI & (~SM2 | rcvbuf[8]))
      begin sbufr=rcvbuf[7:0]; RB8=rcvbuf[8];RI=1; end
    end
  end
end

always @(REN or SM0 or SM1)
begin
  if(REN&~SM0&~SM1&~RI)
  begin
    txdbuf=1;
    @(posedge state6)
    repeat(8)
    begin
      @(posedge state3) txdbuf=0;
      @(posedge state5) @(posedge ph2) sm1=rxd;
      @(posedge state6) rcvbuf[6:0]=rcvbuf[7:1]; rcvbuf[7]=sm1; txdbuf=1;
    end
    @(posedge state1) sbufr=rcvbuf; RI=1;
  end
end

assign p3_0_sel = (state5&ph2&~SM0&~SM1 | SM0 | SM1) & REN;

/************** SERIAL PORT ************************/  
 
/******************* CONTROLLOR ********************/  

assign lastcyc = (cycno==`ONEcyc) | ((cycno==`TWOcyc)&cycle2) | cycle4;

always @(posedge reset or posedge state2)
if(reset)
begin
  here=1; there=1;
  saveAc=0; saveOv=0; ir3_to_a8=0; Cy_to_d1=0; sign=1; p2_sel=1; Ac_to_d8=0;
  p0_to_a8=0; a16bus_src=`PC; dp_l_to_a8=0; dp_h_to_a8=0; 
  Z_reg=0; desCy_to_d1=0; rd_acc=0; wr_acc=0;
  d8_to_a16b_h=0; set_cy=0;
  rd_ram=0;rd_ram_b=0;wr_ram=0;wr_ram_b=0;rd_time=0;wr_time=0;
  sp_add_1=0;sp_to_a8=0;pc_h_to_d8=0;pc_l_to_d8=0;d8_to_pc_h=0;d8_to_pc_l=0;
  ir1_to_d8=0;ir2_to_d8=0;ir3_to_d8=0;ir2_to_a8=0;
  ld_tmp1=0;ld_tmp2_l=0;ld_tmpb=0;des_l_to_d8=0;
  rn_to_a8=0;ri_to_a8=0;d8_to_a16b_l=0;a16b_l_to_a8=0;
  a16b_l_to_d8=0;a16b_h_to_d8=0; des_h_to_d8=0;ld_tmp2_h=0; saveCy=0;
  dirc_flg=0; indirc_flg=0;
end
else
begin
  case(instruction[4:0])

  `AJMP: 
  begin
    @(posedge cycle2) @(posedge state4)
    wr_ir12_to_pc;
  end
  `ACALL:
  begin
    save_pc;
    wr_ir12_to_pc;
  end

  endcase
      
  case (instruction[7:3])

  `MOV_RA:
  begin
    @(posedge state6) rd_acc = 1; movr_wr; rd_acc = 0;
  end

  `MOV_RC:
  begin
    @(posedge cycle2)
    @(posedge state6) ir2_to_d8 = 1; movr_wr; ir2_to_d8 = 0;
  end

  `MOV_RD:
  begin
    @(posedge cycle2) @(posedge state3) movd_rd;
    @(posedge state6)
    a16b_l_to_d8 = 1; movr_wr; a16b_l_to_d8 = 0;
  end

  `MOV_DR:
  begin
    @(posedge cycle2) @(posedge state3) movr_rd;
    @(posedge state6)
    a16b_l_to_d8 = 1; movd_wr; a16b_l_to_d8 = 0;
  end

  `MOV_R:
  begin
    @(posedge state6)
    rd_ram = 1; rn_to_a8 = 1;
    @(posedge ph1) wr_acc = 1;
    @(negedge ph1) wr_acc = 0;
    rd_ram = 0; rn_to_a8 = 0; 
  end

  `XCH_R:
  begin
    srcb=`ONE; alu_op = `ALU_XCH; src1=`ACC; src2=`D8BUS;
    ld_alu_r;
    sv_alu_r;
  end

  `INC_R: inc_dec_r(`ALU_ADD);
  `DEC_R: inc_dec_r(`ALU_SUB);
  `ANL_R: alu_r(`ALU_AND,`D1BUS);
  `ORL_R: alu_r(`ALU_OR,`D1BUS);
  `XRL_R: alu_r(`ALU_XOR,`D1BUS);
  `ADD_R: alu_r(`ALU_ADD,`ZERO);
  `ADDC_R: alu_r(`ALU_ADD,`ONE);
  `SUBB_R: alu_r(`ALU_SUB,`ONE);

  `DJNZ_R:
  begin
    src1 = `D8BUS; src2 = `ZERO; srcb = `ONE; alu_op = `ALU_SUB;
    rd_ram = 1; rn_to_a8 = 1;
    @(posedge ph2) ld_tmpb = 1; ld_tmp1 = 1; ld_tmp2_l = 1;
    @(negedge ph2) ld_tmpb = 0; ld_tmp1 = 0; ld_tmp2_l = 0;
    rd_ram = 0; rn_to_a8 = 0;
    @(posedge state6)
    wr_ram = 1; rn_to_a8 = 1; des_l_to_d8 = 1;
    @(posedge ph2) wr_time = 1;
    @(negedge ph2) wr_time = 0;
    wr_ram = 0; rn_to_a8 = 0; des_l_to_d8 = 0;
    Z_reg = desZ;
    djnz_snd_prt(Z_reg);
  end

  `CJNE_R:
  begin
    src1 = `D8BUS; srcb = `ZERO;
    rd_ram = 1; rn_to_a8 = 1;
    //cycle1, state2
    @(posedge ph2) ld_tmpb = 1; ld_tmp1 = 1;
    @(negedge ph2) ld_tmpb = 0; ld_tmp1 = 0;
    rd_ram = 0; rn_to_a8 = 0;
    @(posedge state4);
    //cycle1, state4
    cjne_snd_prt;
  end

  endcase

  case (instruction[7:1])

  `MOV_DI:
  begin
    @(posedge cycle2)
    movi_r_a;
    movi_r_d;
    @(posedge state6) 
    a16b_h_to_d8=1; movd_wr; a16b_h_to_d8=0; 
  end

  `MOV_I:
  begin
    movi_r_a;
    @(posedge state6) indirc_flg=1;
    rd_ram = 1; a16b_l_to_a8 = 1;
    @(posedge ph1) wr_acc=1;
    @(negedge ph1) wr_acc=0;
    rd_ram = 0; a16b_l_to_a8 = 0; indirc_flg=0;
  end

  `MOV_IA:
  begin
    movi_r_a;
    @(posedge state6)
    rd_acc = 1;
    movi_wr;
    rd_acc = 0;
  end

  `MOV_IC:
  begin
    movi_r_a;
    @(posedge state6)
    ir2_to_d8=1; movi_wr; ir2_to_d8=0;
  end

  `MOV_ID:
  begin
    @(posedge cycle2)
    movi_r_a;
    @(posedge state5) dirc_flg=1;
    rd_ram = 1; ir2_to_a8 = 1;
    @(posedge ph2) d8_to_a16b_h = 1;
    @(negedge ph2) d8_to_a16b_h = 0;
    rd_ram = 0; ir2_to_a8 = 0; dirc_flg=0;
    a16b_h_to_d8=1; movi_wr; a16b_h_to_d8=0;
  end

  `INC_I: inc_dec_i(`ALU_ADD);
  `DEC_I: inc_dec_i(`ALU_SUB);
  `ANL_I: alu_i(`ALU_AND,`D1BUS);
  `ORL_I: alu_i(`ALU_OR,`D1BUS);
  `XRL_I: alu_i(`ALU_XOR,`D1BUS);
  `ADD_I: alu_i(`ALU_ADD,`ZERO);
  `ADDC_I: alu_i(`ALU_ADD,`ONE);
  `SUBB_I: alu_i(`ALU_SUB,`ONE);

  `XCH_I:
  begin
    srcb=`ONE; alu_op = `ALU_XCH; src1=`ACC; src2=`D8BUS;
    ld_alu_i;
    sv_alu_i;
  end

  `XCHD:
  begin
    srcb=`ZERO; alu_op = `ALU_XCH; src1=`ACC; src2=`D8BUS;
    ld_alu_i;
    sv_alu_i;
  end

  `CJNE_I:
  begin
    src1 = `D8BUS; srcb = `ZERO;
    rd_ram = 1; ri_to_a8 = 1;
    //cycle1, state2
    @(posedge ph2) d8_to_a16b_l=1;
    @(negedge ph2) d8_to_a16b_l=0;
    ri_to_a8 = 0; a16b_l_to_a8=1; indirc_flg=1;
    @(posedge ph2) ld_tmpb = 1; ld_tmp1 = 1;
    @(negedge ph2) ld_tmpb = 0; ld_tmp1 = 0;
    rd_ram = 0; a16b_l_to_a8=0; indirc_flg=0;
    //cycle1, state4
    cjne_snd_prt;
  end

  `MOVX_I:
  begin
    movi_r_a;
    @(posedge state5) a16bus_src=`A16BUF; p2_sel=0;
    @(posedge state3) 
    @(posedge ph1) wr_acc = 1;
    @(negedge ph1) wr_acc = 0;
    @(posedge state5) a16bus_src=`PC; p2_sel=1;
  end

  `MOVX_IA:
  begin
    movi_r_a;
    wr_ram=1; p0_to_a8 =1; rd_acc=1; dirc_flg=1;
    @(posedge ph2) wr_time=1;
    @(negedge ph2) wr_time=0;
    wr_ram=0; p0_to_a8 =0; rd_acc=0; dirc_flg=0;
    @(posedge state5) a16bus_src=`A16BUF; p2_sel=0;
    @(posedge cycle2)
    @(posedge state5) a16bus_src=`PC; p2_sel=1;
  end

  endcase

  case (instruction)
  `NOP: ;
  `INC_A: inc_dec_a(`ALU_ADD);
  `DEC_A: inc_dec_a(`ALU_SUB);
  `INC_D: inc_dec_d(`ALU_ADD);
  `DEC_D: inc_dec_d(`ALU_SUB);
  `ANL_C: alu_c(`ALU_AND,`D1BUS);
  `ORL_C: alu_c(`ALU_OR,`D1BUS);
  `XRL_C: alu_c(`ALU_XOR,`D1BUS);
  `ADD_C: alu_c(`ALU_ADD,`ZERO);
  `ADDC_C: alu_c(`ALU_ADD,`ONE);
  `SUBB_C: alu_c(`ALU_SUB,`ONE);
  `ANL_D: alu_d(`ALU_AND,`D1BUS);
  `ORL_D: alu_d(`ALU_OR,`D1BUS);
  `XRL_D: alu_d(`ALU_XOR,`D1BUS);
  `ADD_D: alu_d(`ALU_ADD,`ZERO);
  `ADDC_D: alu_d(`ALU_ADD,`ONE);
  `SUBB_D: alu_d(`ALU_SUB,`ONE);
  `ANL_DC: alu_dc(`ALU_AND,`D8BUS);
  `ORL_DC: alu_dc(`ALU_OR,`D8BUS);
  `XRL_DC: alu_dc(`ALU_XOR,`D8BUS);
  `ANL_DA: alu_dc(`ALU_AND,`ACC);
  `ORL_DA: alu_dc(`ALU_OR,`ACC);
  `XRL_DA: alu_dc(`ALU_XOR,`ACC);
  `CPL_A:
  begin
    @(posedge state5) not_acc=1;
    @(negedge ph1) not_acc=0;
  end
  `CLR_A:
  begin
    @(posedge state5) clr_acc=1;
    @(negedge ph1) clr_acc=0;
  end

  `RL: alu_rt(`ALU_RL);
  `RLC: alu_rt(`ALU_RLC);
  `RR: alu_rt(`ALU_RR);
  `RRC: alu_rt(`ALU_RRC);
  `SWAP: alu_rt(`ALU_SWP);

  `MOV_C:
  begin
    @(posedge state6)
    ir2_to_d8 = 1;
    @(posedge ph1) wr_acc = 1;
    @(negedge ph1) wr_acc = 0;
    ir2_to_d8 = 0; 
  end

  `MOV_D:
  begin
    @(posedge state6) dirc_flg=1;
    rd_ram = 1; ir2_to_a8 = 1;
    @(posedge ph1) wr_acc = 1;
    @(negedge ph1) wr_acc = 0;
    rd_ram = 0; ir2_to_a8 = 0; dirc_flg=0;
  end

  `MOV_DA:
  begin
    @(posedge state6) rd_acc = 1; movd_wr;  rd_acc = 0;
  end

  `MOV_DC:
  begin
    @(posedge cycle2)
    @(posedge state6) ir3_to_d8 = 1; movd_wr;  ir3_to_d8 = 0;
  end

  `MOV_DD:
  begin
    @(posedge cycle2) @(posedge state5) 
    movd_rd; a16b_l_to_d8=1; movd3_wr;  a16b_l_to_d8=0;
  end

  `XCH_D:
  begin
    srcb=`ONE; alu_op = `ALU_XCH; src1=`ACC; src2=`D8BUS;
    ld_alu_d;
    sv_alu_d;
  end

  `MUL:
  begin
    srcb=`ZERO; alu_op = `ALU_MUL; src1=`ACC; src2=`B;
    ld_alu_a;
    @(posedge cycle4) @(posedge state5) 
    des_h_to_d8=1; wr_b=1; saveOv=1;
    @(posedge ph2) des_h_to_d8=0; wr_b=0; saveOv=0;
    sv_acc(`ALU_MUL);
  end

  `DIV:
  begin
    srcb=`ZERO; alu_op = `ALU_DIV; src1=`ACC; src2=`B;
    ld_alu_a;
    @(posedge cycle4) @(posedge state5) 
    des_h_to_d8=1; wr_b=1; saveOv=1;
    @(posedge ph2) des_h_to_d8=0; wr_b=0; saveOv=0;
    sv_acc(`ALU_MUL);
  end

  `DA:
  begin
    srcb=`CY; alu_op = `ALU_DA; src1=`ACC; src2=`D8BUS; Ac_to_d8=1;
    ld_alu_a; Ac_to_d8=0;
    sv_acc(`ALU_DA);
  end

  `DJNZ_D:
  begin
    @(posedge state5) 
    src1 = `D8BUS; src2 = `ZERO; srcb = `ONE; alu_op = `ALU_SUB;
    rd_ram = 1; ir2_to_a8=1; dirc_flg=1;
    @(posedge ph1) ld_tmpb = 1; ld_tmp1 = 1; ld_tmp2_l = 1;
    @(negedge ph1) ld_tmpb = 0; ld_tmp1 = 0; ld_tmp2_l = 0;
    rd_ram = 0; 
    wr_ram = 1; des_l_to_d8 = 1;
    @(posedge ph1) wr_time = 1;
    @(negedge ph1) wr_time = 0;
    wr_ram = 0; ir2_to_a8=0;  des_l_to_d8 = 0; dirc_flg=0;
    Z_reg = desZ;
    cjne_thd_prt(Z_reg);
  end

  `CJNE_C:
  begin
    //cycle1, state2
    src1 = `ACC; srcb = `ZERO;
    @(posedge ph2) ld_tmpb = 1; ld_tmp1 = 1;
    @(negedge ph2) ld_tmpb = 0; ld_tmp1 = 0;
    @(posedge state4)
    //cycle1, state4
    cjne_snd_prt;
  end

  `CJNE_D:
  begin
    src1 = `ACC; srcb = `ZERO; 
    @(posedge ph2) ld_tmpb = 1; ld_tmp1 = 1;
    @(negedge ph2) ld_tmpb = 0; ld_tmp1 = 0; 
    @(posedge state5) 
    src2 = `D8BUS; alu_op = `ALU_SUB;
    rd_ram = 1; ir2_to_a8 = 1; dirc_flg=1;
    @(posedge ph2) ld_tmp2_l = 1;
    @(negedge ph2) ld_tmp2_l = 0;
    rd_ram =0; ir2_to_a8=0; dirc_flg=0;
    @(posedge state6)
    @(posedge ph2) saveCy = 1; Z_reg = desZ;
    @(negedge ph2) saveCy = 0;
    cjne_thd_prt(Z_reg);
  end

  `JB: 
  begin 
    srcb=`D1BUS; ld_bit; @(posedge state1) cjne_thd_prt(tmpb);
  end
  `JNB:
  begin 
    srcb=`D1BUS; ld_bit; @(posedge state1) cjne_thd_prt(~tmpb);
  end
  `JBC:
  begin
    srcb=`D1BUS; ld_bit; @(posedge state1) cjne_thd_prt(tmpb);
    alu_op=`NOP; // in cjne_thd selb=`ZERO, So,desCy=tmpb, So, desCy=0!!
    wr_ram_b =1; ir2_to_a8=1; desCy_to_d1=1; dirc_flg=1;           // wr 0 to bit!!
    @(posedge ph2) wr_time =1;                         		   // at cycle2 state6
    @(negedge ph2) wr_time =0;
    wr_ram_b =0; ir2_to_a8=0; desCy_to_d1=0; dirc_flg=0;
  end

  `JC: @(posedge state4) @(posedge state1) djnz_snd_prt(Cy);
  `JNC: @(posedge state4) @(posedge state1) djnz_snd_prt(~Cy);
  `JZ: @(posedge state4) @(posedge state1) djnz_snd_prt(~accZ);
  `JNZ: @(posedge state4) @(posedge state1) djnz_snd_prt(accZ);

  `ANL_B: bit_op(`ALU_AND,`D1BUS);
  `ANL_NB: bit_op(`ALU_AND,`D1_N);
  `ORL_B: bit_op(`ALU_OR,`D1BUS);
  `ORL_NB: bit_op(`ALU_OR,`D1_N);
  `CPL_B: bit_op1(`ALU_NOP,`D1_N);
  `CLR_B: bit_op1(`ALU_NOP,`ZERO);
  `SETB_B: bit_op1(`ALU_NOP,`ONE);
  `SETB_C:
  begin
    @(posedge state5) set_cy=1;
    @(negedge ph1) set_cy=0;
  end
  `CPL_C:
  begin
    @(posedge state5) not_cy=1;
    @(negedge ph1) not_cy=0;
  end
  `CLR_C:
  begin
    @(posedge state5) clr_cy=1;
    @(negedge ph1) clr_cy=0;
  end

  `MOV_CB:
  begin
    srcb=`D1BUS; alu_op=`NOP; ld_bit; sv_cy;
  end

  `MOV_BC:
  begin
    Cy_to_d1=1; sv_bit; Cy_to_d1=0;
  end

  `PUSH:
  begin
    @(posedge cycle2)                         			// state1
    @(posedge ph2) sp_add_1 = 1;
    @(negedge ph2) sp_add_1 = 0;
    movd_rd; dirc_flg=1;
    wr_ram = 1; a16b_l_to_d8 = 1; sp_to_a8 = 1; 			// state2
    @(posedge ph2) wr_time = 1;               
    @(negedge ph2) wr_time = 0;
    wr_ram = 0; a16b_l_to_d8 = 0; sp_to_a8 = 0; dirc_flg=0;
  end

  `POP:
  begin
    @(posedge cycle2) ;
    @(posedge state2) dirc_flg=1; dirc_flg=1;                         // state2
    sp_to_a8 = 1; rd_ram = 1;
    @(posedge ph2) d8_to_a16b_l = 1;
    @(negedge ph2) d8_to_a16b_l = 0;
    sp_to_a8 = 0; rd_ram = 0;
    @(posedge ph2) sp_sub_1 = 1; 		// state5
    @(negedge ph2) sp_sub_1 = 0;
    a16b_l_to_d8=1; ir2_to_a8=1;
    movd_wr;
    a16b_l_to_d8=0; ir2_to_a8=0;
  end
    
  `RET: retn;
  `RETI:
  begin
    reti_flg=1; retn;
    @(posedge ph2) crt_int=4'b1111; reti_flg=0;
  end

  `LCALL: 
  begin
    save_pc; wr_ir23_to_pc;
  end

  `LJMP :
  begin
    @(posedge cycle2)
    @(posedge state4)
    wr_ir23_to_pc;
  end

  `JMP:
  begin
    @(posedge cycle2) ;
    @(posedge state2) 
    src1 = `ACC; src2 = `D8BUS; srcb = `ZERO; alu_op=`ALU_ADD; sign=0;
    rd_ram=1; dp_l_to_a8=1;
    @(posedge ph2) ld_tmp1=1; ld_tmp2_l=1; ld_tmpb=1;
    @(negedge ph2) ld_tmp1=0; ld_tmp2_l=0; ld_tmpb=0;
    dp_l_to_a8=0;  dp_h_to_a8=1;
    @(posedge ph2) ld_tmp2_h=1;
    @(negedge ph2) ld_tmp2_h=0; 
    rd_ram=0; dp_h_to_a8=0;
    wr_des_to_pc; sign=1;
  end

  `SJMP:
  begin
    @(posedge cycle2)
    src1 =`D8BUS; srcb=`ZERO; alu_op=`ALU_ADD;
    ir2_to_d8=1;
    @(posedge ph2) ld_tmp1= 1; ld_tmpb= 1;
    @(negedge ph2) ld_tmp1= 0; ld_tmpb= 0;
    ir2_to_d8= 0;
    src2 = `D8BUS; pc_l_to_d8 = 1;
    @(posedge ph2) ld_tmp2_l= 1; 
    @(negedge ph2) ld_tmp2_l = 0;
    pc_l_to_d8 = 0; pc_h_to_d8 = 1;
    @(posedge ph2) ld_tmp2_h = 1;
    @(negedge ph2) ld_tmp2_h = 0;
    pc_h_to_d8 = 0;
    wr_des_to_pc;
  end

  `INC_DP:
  begin
    @(posedge cycle2)
    alu_op = `ALU_ADD; src1=`ZERO; src2=`D8BUS; srcb=`ONE;
    dp_l_to_a8=1; rd_ram=1;
    @(posedge ph2)
    ld_tmpb =1; ld_tmp1 =1; ld_tmp2_l =1;
    @(negedge ph2)
    ld_tmpb =0; ld_tmp1 =0; ld_tmp2_l =0;
    dp_l_to_a8=0;  dp_h_to_a8=1;
    @(posedge ph2) ld_tmp2_h =1;
    @(negedge ph2) ld_tmp2_h =0;
    rd_ram=0; wr_ram=1; des_l_to_d8=1; dp_l_to_a8=1; dp_h_to_a8=0;
    @(posedge ph2) wr_time=1;
    @(negedge ph2) wr_time=0;
    des_h_to_d8=1; dp_h_to_a8=1; dp_l_to_a8=0; des_l_to_d8=0;
    @(posedge ph2) wr_time=1;
    @(negedge ph2) wr_time=0; des_h_to_d8=0; dp_h_to_a8=0;
  end
 
  `MOV_DPC:
  begin
    @(posedge cycle2) dirc_flg=1;
    @(posedge state2) dp_l_to_a8=1; wr_ram=1;ir3_to_d8=1; // state2
    @(posedge ph2) wr_time=1;
    @(negedge ph2) dp_l_to_a8=0; wr_time=0; ir3_to_d8=0;
    @(posedge state3) dp_h_to_a8=1; ir2_to_d8=1; // state3
    @(posedge ph2) wr_time=1;
    @(negedge ph2) dp_h_to_a8=0; wr_ram=0; wr_time=0; ir2_to_d8=0; dirc_flg=0;
  end

  `MOVX_AP:
  begin
    @(posedge state5) a16bus_src=`DP;
    @(posedge state3) 
    @(posedge ph1) wr_acc = 1;
    @(negedge ph1) wr_acc = 0;
    @(posedge state5) a16bus_src=`PC;
  end

  `MOVC_DP:
  begin
    @(posedge state3) 
    src1 = `ACC; src2 = `D8BUS; srcb = `ZERO; alu_op=`ALU_ADD; sign=0; 
    dp_l_to_a8=1; rd_ram=1; dirc_flg=1;
    @(posedge ph2) ld_tmp1=1; ld_tmp2_l=1; ld_tmpb=1;
    @(negedge ph2) ld_tmp1=0; ld_tmp2_l=0; ld_tmpb=0;
    dp_l_to_a8=0;  dp_h_to_a8=1;
    @(posedge ph2) ld_tmp2_h=1;
    @(negedge ph2) ld_tmp2_h=0; dp_h_to_a8=0; rd_ram=0;  dirc_flg=0;
    des_l_to_d8=1; 
    @(posedge ph2) d8_to_a16b_l=1;
    @(negedge ph2) d8_to_a16b_l=0; des_l_to_d8=0; des_h_to_d8=1;
    @(posedge ph2) d8_to_a16b_h=1;
    @(negedge ph2) d8_to_a16b_h=0; des_h_to_d8=0; a16bus_src=`A16BUF; sign=1;
    @(posedge state4) 
    @(posedge ph1) wr_acc=1;
    @(negedge ph1) wr_acc=0; a16bus_src=`PC;
  end

  `MOVC_PC:
  begin
    @(posedge state3) 
    src1 = `ACC; src2 = `D8BUS; srcb = `ZERO; alu_op=`ALU_ADD; sign=0; 
    pc_l_to_d8=1;  
    @(posedge ph2) ld_tmp1=1; ld_tmp2_l=1; ld_tmpb=1;
    @(negedge ph2) ld_tmp1=0; ld_tmp2_l=0; ld_tmpb=0;
    pc_l_to_d8=0;  pc_h_to_d8=1;
    @(posedge ph2) ld_tmp2_h=1;
    @(negedge ph2) ld_tmp2_h=0; pc_h_to_d8=0; 
    des_l_to_d8=1; 
    @(posedge ph2) d8_to_a16b_l=1;
    @(negedge ph2) d8_to_a16b_l=0; des_l_to_d8=0; des_h_to_d8=1;
    @(posedge ph2) d8_to_a16b_h=1;
    @(negedge ph2) d8_to_a16b_h=0; des_h_to_d8=0; a16bus_src=`A16BUF; sign=1;
    @(posedge state4) 
    @(posedge ph1) wr_acc=1;
    @(negedge ph1) wr_acc=0; a16bus_src=`PC;
  end

  `MOVX_PA:
  begin
    @(posedge state3) 
    wr_ram=1; p0_to_a8 =1; rd_acc=1; dirc_flg=1;
    @(posedge ph2) wr_time=1;
    @(negedge ph2) wr_time=0;
    wr_ram=0; p0_to_a8 =0; rd_acc=0; dirc_flg=0;
    @(posedge state5) a16bus_src=`DP;
    @(posedge cycle2)
    @(posedge state5) a16bus_src=`PC;
  end

  endcase

end

assign p0_to_d8 = (cycle1 & state1 & ph1)
      		| (cycle1 & state4 & ph1 & (i_byte==2 | i_byte==3))
      		| (cycle2 & state1 & ph1 & (i_byte==3))
      		| (cycle2 & state3 & ph1 & ~x_rd)
      		| (cycle2 & state4 & ph1 & (instruction==`MOVC_DP|instruction==`MOVC_PC));
assign a8bus = dp_h_to_a8? `SFR_DPTR_HI : 8'bz;
assign a8bus = dp_l_to_a8? `SFR_DPTR_LO : 8'bz;
assign d8bus[0] = Ac_to_d8? Ac : 1'bz;
  
assign a8bus = p0_to_a8? `SFR_P0 : 8'bz;
      
//$monitor("here%d: ",here,$time,"i_byte: %d", i_byte); 
    
always @(posedge state2 or posedge reset)
  if(reset) i_byte = `ONEbyte;
  else
  begin
    case(instruction[4:0])
    `ACALL,
    `AJMP:
      i_byte = `TWObyte;
    endcase
  end

always @(posedge state2)
begin
  case(instruction[7:3])
  `CJNE_R: 
    i_byte = `THREEbyte;
  `MOV_DR,`DJNZ_R,`MOV_RC,
  `MOV_RD:
    i_byte = `TWObyte;
  `INC_R,`DEC_R,`ADD_R,`ADDC_R,`ORL_R,
  `ANL_R,`XRL_R,`SUBB_R,`XCH_R,`MOV_R,
  `MOV_RA:
    i_byte = `ONEbyte;
  endcase
end

always @(posedge state2)
begin
  case(instruction[7:1])
  `CJNE_I:
   i_byte = `THREEbyte;
  `MOV_DI, `MOV_ID,
  `MOV_IC:
    i_byte = `TWObyte;
  `MOV_I,`INC_I,`DEC_I,`ADD_I,`ADDC_I,`ORL_I,
  `ANL_I,`XRL_I,`SUBB_I,`XCH_I,`XCHD,
  `MOV_IA,`MOVX_I,
  `MOVX_IA:
    i_byte = `ONEbyte;
  endcase
end

always @(posedge state2)
begin
  case(instruction)
  `LCALL,`DJNZ_D,`CJNE_D,`CJNE_C,`MOV_DPC,`MOV_DD,
  `MOV_DC,`XRL_DC,`XRL_DC,`ORL_DC,`JNB,`JB,`JBC,
  `LJMP:
    i_byte = `THREEbyte;
  `INC_D,`DEC_D,`ADD_C,`ADD_D,`ADDC_C,`ADDC_D,
  `ORL_DA,`ORL_C,`ORL_D,`ANL_DA,`ANL_C,`ANL_D,
  `XRL_DA,`XRL_C,`XRL_D,`JC,`JNC,`JZ,`JNZ,`ORL_B,
  `MOV_C,`SJMP,`ANL_B,`MOV_BC,`SUBB_C,`SUBB_D,
  `ORL_NB,`MOV_CB,`ANL_NB,`CPL_B,`PUSH,`CLR_B,
  `XCH_D,`SETB_B,`MOV_D,`MOV_DA,
  `POP: 
    i_byte = `TWObyte;
  `NOP,`RR,`INC_A,`RRC,`DEC_A,`RET,`RL,`RETI,`RLC,
  `JMP,`MOVC_PC,`DIV,`MOVC_DP,`INC_DP,`MUL,`CPL_C,
  `CLR_C,`SWAP,`SETB_C,`DA,`MOVX_AP,`CLR_A,`MOVX_PA,
  `CPL_A:
    i_byte = `ONEbyte;
  endcase
end
      
always @(posedge state2 or posedge reset)
  if(reset) cycno = `ONEcyc;
  else
  begin
    case(instruction[4:0])
    `ACALL,
    `AJMP:
      cycno = `TWOcyc;
    endcase
  end

always @(posedge state2)
begin
  case(instruction[7:3])
  `CJNE_R, `DJNZ_R, `MOV_DR,`MOV_RC,
  `MOV_RD: 
    cycno = `TWOcyc;
  `INC_R,`DEC_R,`ADD_R,`ADDC_R,`ORL_R,
  `ANL_R,`XRL_R,`SUBB_R,`XCH_R,`MOV_R,
  `MOV_RA:
    cycno = `ONEcyc;
  endcase
end

always @(posedge state2)
begin
  case(instruction[7:1])
  `CJNE_I, `MOV_DI, `MOV_ID, `MOVX_I,
  `MOVX_IA: 
    cycno = `TWOcyc;
  `MOV_I,`INC_I,`DEC_I,`ADD_I,`ADDC_I,`ORL_I,
  `ANL_I,`XRL_I,`SUBB_I,`XCH_I,`XCHD,`MOV_IA,
  `MOV_IC:
    cycno = `ONEcyc;
  endcase
end

always @(posedge state2)
begin
  case(instruction)
  `MUL,
  `DIV:
    cycno = `FOURcyc;
  `LJMP,`JBC,`LCALL,`JB,`RET,`JNB,`RETI,`JC,`ORL_DC,
  `JNC,`ANL_DC,`JZ,`XRL_DC,`JNZ,`JMP,`MOV_DC,`SJMP,
  `MOVC_PC,`MOV_DD,`MOV_DPC,`MOVC_DP,`CJNE_C,`CJNE_D,
  `PUSH,`POP,`DJNZ_D,`MOVX_AP,`INC_DP,
  `MOVX_PA:
    cycno = `TWOcyc;
  `NOP,`RR,`INC_A,`INC_D,`RRC,`DEC_A,`DEC_D,`RL,
  `ADD_C,`ADD_D,`RLC,`ADDC_C,`ADDC_D,`ORL_DA,`ORL_C,
  `ORL_D,`ANL_DA,`ANL_C,`ANL_D,`XRL_DA,`XRL_C,`XRL_D,
  `ORL_B,`MOV_C,`ANL_B,`MOV_BC,`SUBB_C,`SUBB_D,
  `ORL_NB,`MOV_CB,`ANL_NB,`CPL_B,`CPL_C,
  `CLR_B,`CLR_C,`SWAP,`XCH_D,`SETB_B,`SETB_C,`DA,
  `CLR_A,`MOV_D,`CPL_A,
  `MOV_DA:
    cycno = `ONEcyc;
  endcase
end
      
always @(negedge state6 or posedge reset)
begin
  if(reset)  cycle = `FIRST;
  else
  begin
    case (cycle)
    `FIRST:// to cycle01 or cycle00
      case(cycno)
      `FOURcyc,
      `TWOcyc: cycle = #1 `SECOND;
      default: cycle = #1 `FIRST; 
      endcase
    `SECOND://to cycle00, except MUL or DIV;
      case(cycno)
      `FOURcyc: cycle = `THIRD;
      default: cycle = #1 `FIRST;
      endcase
    `THIRD: cycle = `FORTH;
    `FORTH: cycle = `FIRST;
    endcase
  end
end
      
always @(posedge state3 or posedge reset)
begin
  if(reset)
  begin
    x_wr=1; x_rd=1; ext_rw=1;
  end
  else if(instruction[7:1]==`MOVX_I | instruction==`MOVX_AP)
  begin
    @(posedge state5) ext_rw = 0;
    @(posedge state1) x_rd = 0;
    @(posedge state4) x_rd = 1;
    @(posedge state5) ext_rw = 1;
  end
  else if(instruction[7:1]==`MOVX_IA | instruction==`MOVX_PA)
  begin
    @(posedge state5) ext_rw = 0;
    @(posedge state1) x_wr = 0;
    @(posedge state4) x_wr = 1;
    @(posedge state5) ext_rw = 1;
  end
end

task alu_rt;
input [3:0] op;
begin
  alu_op = op; src1=`ACC; src2=`ZERO; srcb=`ZERO;
  ld_alu_a;
  sv_acc(op);
end
endtask

task djnz_snd_prt;
input flag;
begin
  if(flag == 1)
  begin
    alu_op = `ALU_ADD;
    ir2_to_d8 = 1; src1 = `D8BUS; srcb = `ZERO;
    @(posedge ph2) ld_tmp1 = 1; ld_tmpb = 1;
    @(negedge ph2) ld_tmp1 = 0; ld_tmpb = 0;
    ir2_to_d8 = 0;
    pc_plus_rel;
  end
  else
    @(posedge state6) ;
end
endtask

task cjne_snd_prt;
begin
  //cycle1, state4
  src2 = `D8BUS; alu_op = `ALU_SUB;
  @(posedge ph1) ld_tmp2_l = 1;  // data enter ir2 this time also !
  @(negedge ph1) ld_tmp2_l = 0; 
  //cycle1, state6
  @(posedge state6)
  @(posedge ph2) saveCy = 1; Z_reg = desZ;
  @(negedge ph2) saveCy = 0;
  cjne_thd_prt(Z_reg);
end
endtask

task cjne_thd_prt;
input flag;
begin
  if(flag == 1)
  begin
    alu_op = `ALU_ADD;
    src1 = `D8BUS; srcb = `ZERO;    //cycle2, state1
    @(posedge ph1) ld_tmpb = 1; ld_tmp1 = 1; // data enter ir3 this time also !
    @(negedge ph1) ld_tmp1 = 0; ld_tmpb = 0;
    @(posedge state2)
    pc_plus_rel;
  end
  else
    @(posedge state4) ;
end
endtask

task pc_plus_rel;
begin
  src2 = `D8BUS;
  //cycle2, state2
  pc_l_to_d8 = 1;
  @(posedge ph2) ld_tmp2_l = 1;
  @(negedge ph2) ld_tmp2_l = 0; pc_l_to_d8 = 0; 
  //cycle2, state3
  pc_h_to_d8 = 1;
  @(posedge ph2) ld_tmp2_h = 1;
  @(negedge ph2) ld_tmp2_h = 0; pc_h_to_d8 = 0;
  //cycle2, state4
  des_l_to_d8 = 1;
  @(posedge ph2) d8_to_pc_l = 1;
  @(negedge ph2) d8_to_pc_l = 0; des_l_to_d8 = 0;
  //cycle2, state5
  des_h_to_d8 = 1;
  @(posedge ph2) d8_to_pc_h = 1;
  @(negedge ph2) d8_to_pc_h = 0; des_h_to_d8 = 0;
end
endtask

task Cy_op;
input [3:0] op;
input sel;
begin
  alu_op=op; srcb=sel;
  @(posedge state5) 
  rd_ram_b = 1; ir2_to_a8=1; 
  @(posedge ph2) ld_tmpb =1;
  @(negedge ph2) ld_tmpb =0; 
  rd_ram_b =0; wr_ram_b = 1; desCy_to_d1=1;
  @(posedge ph2) wr_time =1;
  @(negedge ph2) wr_time =0; 
  wr_ram_b =0; ir2_to_a8=0; desCy_to_d1=0;
end
endtask

task ld_bit;
begin
  @(posedge state5) rd_ram_b = 1; ir2_to_a8=1; dirc_flg=1;
  @(posedge ph2) ld_tmpb =1;
  @(negedge ph2) ld_tmpb =0; rd_ram_b =0; ir2_to_a8=0; dirc_flg=0;
end
endtask

task sv_bit;
begin
    wr_ram_b = 1; ir2_to_a8=1; dirc_flg=1;
    @(posedge ph2) wr_time =1;
    @(negedge ph2) wr_time =0; wr_ram_b =0; ir2_to_a8=0; dirc_flg=0;
end
endtask

task sv_cy;
begin
  @(posedge ph2) saveCy =1;
  @(negedge ph2) saveCy =0;
end
endtask

task bit_op;
input [3:0] op;
input [2:0] sel;
begin
  alu_op=op; srcb=sel; ld_bit; sv_cy;
end
endtask

task bit_op1;
input [3:0] op;
input [2:0] sel;
begin
  alu_op=op; srcb=sel; ld_bit; 
  desCy_to_d1=1; sv_bit; desCy_to_d1=0;
end
endtask

task movi_r_d;
begin
  rd_ram = 1; a16b_l_to_a8 = 1; indirc_flg=1;
  @(posedge ph2) d8_to_a16b_h=1;
  @(negedge ph2) d8_to_a16b_h=0;
  rd_ram = 0; a16b_l_to_a8 = 0; indirc_flg=0;
end
endtask

task movi_r_a;
begin
  rd_ram = 1; ri_to_a8 = 1;
  @(posedge ph2) d8_to_a16b_l = 1;
  @(negedge ph2) d8_to_a16b_l = 0;
  rd_ram = 0; ri_to_a8 = 0;
end
endtask

task movi_wr;
begin
  wr_ram = 1; a16b_l_to_a8 = 1; indirc_flg=1;
  @(posedge ph2) wr_time = 1;
  @(negedge ph2) wr_time = 0;
  wr_ram = 0; a16b_l_to_a8 = 0; indirc_flg=0;
end
endtask

task movr_rd;
begin
  rd_ram = 1; rn_to_a8 = 1;
  @(posedge ph2) d8_to_a16b_l = 1;
  @(negedge ph2) d8_to_a16b_l = 0;
  rd_ram = 0; rn_to_a8 = 0;
end
endtask

task movr_wr;
begin
  wr_ram = 1; rn_to_a8 = 1;
  @(posedge ph2) wr_time = 1;
  @(negedge ph2) wr_time = 0;
  wr_ram = 0; rn_to_a8 = 0;
end
endtask

task movd_rd;
begin
  rd_ram = 1; ir2_to_a8 = 1; dirc_flg=1;
  @(posedge ph2) d8_to_a16b_l = 1;
  @(negedge ph2) d8_to_a16b_l = 0;
  rd_ram = 0; ir2_to_a8 = 0; dirc_flg=0;
end
endtask

task movd_wr;
begin
  wr_ram = 1; ir2_to_a8 = 1; dirc_flg=1;
  @(posedge ph2) wr_time = 1;
  @(negedge ph2) wr_time = 0;
  wr_ram = 0; ir2_to_a8 = 0; dirc_flg=0; 
end
endtask

task movd3_wr;
begin
  wr_ram = 1; ir3_to_a8 = 1; dirc_flg=1;
  @(posedge ph2) wr_time = 1;
  @(negedge ph2) wr_time = 0;
  wr_ram = 0; ir3_to_a8 = 0;dirc_flg=0; 
end
endtask

task alu_i;
input [3:0] op;
input [2:0] b_sel;
begin
  srcb = b_sel; alu_op = op; src1=`ACC; src2=`D8BUS;
  ld_alu_i;
  sv_acc(op);
end
endtask

task alu_r;
input [3:0] op;
input [2:0] b_sel;
begin
  srcb = b_sel; alu_op = op; src1=`ACC; src2=`D8BUS;
  ld_alu_r;
  sv_acc(op);
end
endtask

task ld_alu_d;
begin
  dirc_flg=1;
  @(posedge state5) ir2_to_a8 = 1; ld_alu; ir2_to_a8 = 0;
  dirc_flg=0;
end
endtask

task ld_alu_r;
begin
  @(posedge state5) rn_to_a8 = 1; ld_alu; rn_to_a8 = 0;
end
endtask

task ld_alu_i;
begin
  movi_r_a; indirc_flg=1;
  @(posedge state5) a16b_l_to_a8 = 1; ld_alu; a16b_l_to_a8 = 0;
  indirc_flg=0;
end
endtask

task ld_alu;
begin
  rd_ram = 1; ld_alu_a; rd_ram = 0;
end
endtask

task ld_alu_a;
begin
  @(posedge ph2) ld_tmpb = 1; ld_tmp1 = 1; ld_tmp2_l = 1;
  @(negedge ph2) ld_tmpb = 0; ld_tmp1 = 0; ld_tmp2_l = 0;
end
endtask

task sv_acc;
input [3:0] op;
begin
  @(posedge state6)
  des_l_to_d8 = 1;
  @(posedge ph1) wr_acc = 1; 
  if(op==`ALU_ADD | op==`ALU_SUB | op==`ALU_MUL | op==`ALU_DIV | op==`ALU_DA) saveCy=1;
  @(negedge ph1) wr_acc = 0;
  des_l_to_d8 = 0; saveCy=0;
end
endtask

task sv_alu_i;
begin
  sv_acc(`ALU_NOP); indirc_flg=1;
  a16b_l_to_a8 = 1; sv_des_h_to_ram; a16b_l_to_a8 = 0;
  indirc_flg=0;
end
endtask

task sv_alu_r;
begin
  sv_acc(`ALU_NOP); rn_to_a8 = 1; sv_des_h_to_ram; rn_to_a8 = 0;
end
endtask

task sv_alu_d;
begin
  dirc_flg=1;
  sv_acc(`ALU_NOP); ir2_to_a8 = 1; sv_des_h_to_ram; ir2_to_a8 = 0;
  dirc_flg=0;
end
endtask

task sv_des_h_to_ram;
begin
  wr_ram = 1; des_h_to_d8 = 1;
  wr_time = 1;
  @(negedge ph2) wr_time = 0;
  wr_ram = 0; des_h_to_d8 = 0;
end
endtask

task inc_dec_a;
input [3:0] op;
begin
  src1=`ACC; src2=`ZERO; srcb = `ONE; alu_op =op;
  ld_alu_a;
  sv_des_l_to_ram;
end
endtask

task inc_dec_d;
input [3:0] op;
begin
  srcb = `ONE; alu_op =op; src1=`D8BUS; src2=`ZERO;
  ld_alu_d;
  ir2_to_a8 = 1; sv_des_l_to_ram; ir2_to_a8 = 0;
end
endtask

task inc_dec_r;
input [3:0] op;
begin
  srcb = `ONE; alu_op =op; src1=`D8BUS; src2=`ZERO;
  ld_alu_r;
  rn_to_a8 = 1; sv_des_l_to_ram; rn_to_a8 = 0;
end
endtask

task inc_dec_i;
input [3:0] op;
begin
  srcb = `ONE; alu_op =op; src1=`D8BUS; src2=`ZERO;
  ld_alu_i; indirc_flg=1;
  a16b_l_to_a8 = 1; sv_des_l_to_ram; a16b_l_to_a8 = 0;
  indirc_flg=0;
end
endtask

task sv_des_l_to_ram;
begin
//  @(posedge state6)
  wr_ram = 1; des_l_to_d8 = 1;
  @(posedge ph2) wr_time = 1;
  @(negedge ph2) wr_time = 0;
  wr_ram = 0; des_l_to_d8 = 0;
end
endtask

task alu_c;
input [3:0] op;
input [2:0] b_sel;
begin
  srcb = b_sel; alu_op = op; src1=`ACC; src2=`D8BUS;
  @(posedge state4)
  ld_tmpb = 1; ld_tmp1 = 1; ld_tmp2_l = 1;
  @(negedge ph1) ld_tmpb = 0; ld_tmp1 = 0; ld_tmp2_l = 0;
  sv_acc(op);
end
endtask

task alu_d;
input [3:0] op;
input [2:0] b_sel;
begin
  srcb = b_sel; alu_op = op; src1=`ACC; src2=`D8BUS;
  ld_alu_d;
  sv_acc(op);
end
endtask

task alu_dc;
input [3:0] op;
input [1:0] srcsel;
begin
  srcb=`ZERO; alu_op =op; src1=`ACC; src2=`D8BUS;
  ld_alu_d;
  @(posedge state1) src1=srcsel;
  @(posedge ph1) ld_tmp1 = 1;
  @(negedge ph1) ld_tmp1 = 0;
  @(posedge state6) des_l_to_d8 = 1; movd_wr;  des_l_to_d8 = 0;
end
endtask

task wr_ir12_to_pc;
begin
  ir2_to_d8 = 1;
  @(posedge ph1) d8_to_pc_l = 1;
  @(negedge ph1) d8_to_pc_l = 0;
  ir2_to_d8 = 0; ir1_to_d8 = 1; 
  d8_to_pc_h = 1;
  @(negedge ph2) d8_to_pc_h = 0;
  ir1_to_d8 = 0;
end
endtask

task wr_ir23_to_pc;
begin
  ir3_to_d8 = 1;
  @(posedge ph1) d8_to_pc_l = 1;
  @(negedge ph1) d8_to_pc_l = 0;
  ir3_to_d8 = 0; ir2_to_d8 = 1; 
  d8_to_pc_h = 1;
  @(negedge ph2) d8_to_pc_h = 0;
  ir2_to_d8 = 0;
end
endtask

task wr_des_to_pc;
begin
  des_l_to_d8=1;
  @(posedge ph2) d8_to_pc_l=1;
  @(negedge ph2) d8_to_pc_l=0;
  des_l_to_d8=0; des_h_to_d8=1;
  @(posedge ph2) d8_to_pc_h=1;
  @(negedge ph2) d8_to_pc_h=0;
  des_h_to_d8=0;
end
endtask

task retn;
begin
  @(posedge cycle2)
  @(posedge state2)                         // state2
  sp_to_a8 = 1; rd_ram = 1;
  @(posedge ph2) d8_to_pc_h = 1;
  @(negedge ph2) d8_to_pc_h = 0;
  @(posedge ph2) sp_sub_1 = 1; 		// state3
  @(negedge ph2) sp_sub_1 = 0;
  @(posedge ph2) d8_to_pc_l = 1;            // state4
  @(negedge ph2) d8_to_pc_l = 0;
  @(posedge ph2) sp_sub_1 = 1; 		// state5
  @(negedge ph2) sp_sub_1 = 0;
  sp_to_a8 = 0; rd_ram = 0;
end
endtask

task save_pc;
begin
  @(posedge cycle2)                         			// state1
  @(posedge ph2) sp_add_1 = 1;
  @(negedge ph2) sp_add_1 = 0;
  wr_ram = 1; pc_l_to_d8 = 1; sp_to_a8 = 1; 			// state2
  @(posedge ph2) wr_time = 1;               
  @(negedge ph2) wr_time = 0;
  pc_l_to_d8 = 0; sp_add_1 = 1;			// state3
  pc_h_to_d8 = 1; 
  @(posedge ph2) wr_time = 1; sp_add_1 = 0;  
  @(negedge ph2) wr_time = 0;
  wr_ram = 0; pc_h_to_d8 = 0; sp_to_a8 = 0;
end
endtask

/******************* CONTROLLOR *****************/  
 
/******************* INTERRUPT ******************/

reg [3:0] int_buf;     

always @(posedge state5 or posedge reset)
if(reset)
begin
  crt_int=4'b1111; int_i=0; int_r=0; reti_flg=0; int_buf=4'b1111;
end
else
begin
  if(new_int == 4'b1111) 
  begin
    int_i=0; int_buf=new_int;
  end
  else if(int_i==0) 
    @(posedge state6) int_i=1; int_buf=new_int;
end

always @(posedge state5)
  @(posedge ph2)
  if((int_i==1) & (reti_flg==0) & lastcyc & (int_buf<crt_int) & ie_flg==0 & ip_flg==0) 
  begin
    int_r=1; int_i=0; crt_int = int_buf; 
    @(posedge state1)                         // state1
    @(posedge ph2) sp_add_1 = 1; 		
    @(negedge ph2) sp_add_1 = 0;
    pc_l_to_d8 = 1; sp_to_a8 = 1; wr_ram = 1; //state2
    @(posedge ph2) wr_time = 1;
    @(negedge ph2) 
    pc_l_to_d8 = 0; wr_time = 0; sp_add_1 = 1;// state3
    pc_h_to_d8 = 1; 
    @(posedge ph2) wr_time = 1; sp_add_1 = 0;  // state3
    @(negedge ph2) 
    pc_h_to_d8 = 0; sp_to_a8 = 0; wr_time = 0; wr_ram = 0;
    int_r=0;
  end

always @(ir2 or ir3 or reset)
  if(reset) begin ip_flg=0; ie_flg=0; end
  else if( ((ir2==`SFR_IE | ir2==`SFR_IP) & 
            (ir1==`MOV_DC | ir1[7:1]==`MOV_DI | ir1==`MOV_DA | ir1[7:3]==`MOV_DR | 
             ir1==`ANL_DA | ir1==`ANL_DC | ir1==`ORL_DA | ir1==`ORL_DC | 
             ir1==`XRL_DA | ir1==`XRL_DC | ir1==`INC_D | ir1==`DEC_D | ir1==`DJNZ_D)) 
         | ((ir3==`SFR_IE | ir3==`SFR_IP) & (ir1==`MOV_DD))
	 | ((ir2[7:3]==`SFR_B_IE | ir2[7:3]==`SFR_B_IP) & 
            (ir1==`CPL_B | ir1==`SETB_B | ir1==`CLR_B | ir1==`MOV_BC)) ) 
    begin
      // $monitor("time%d",$time,"abuf%h",a16buf[7:0],"instrc%h",instruction);
      ip_flg=1;
      @(posedge state6)
      if(lastcyc) ip_flg=0;
      else @(posedge state1) @(posedge state6) ip_flg=0;
    end

wire [7:0] int_flg;
wire [15:0] py;
reg notfinish; 
 
always @(negedge int0)
  IE0=1;

always @(negedge int1)
  IE1=1;


assign int_flg[0]= IT0 ? IE0 : ~int0;
assign int_flg[1]= TF0;
assign int_flg[2]= IT1 ? IE1 : ~int1;
assign int_flg[3]= TF1;
assign int_flg[4]= RI | TI;
assign int_flg[5]= notfinish;
assign int_flg[6]= notfinish;
assign int_flg[7]= notfinish;
     
assign py[15:8]= ie & (~ip) & int_flg & (EA?'hff:'h00);
assign py[7:0]= ie & ip & int_flg & (EA?'hff:'h00);
  
always @(py or reset)
  if(reset) 
  begin 
    new_int=4'b1111; notfinish =0;
  end
  else
    new_int = py[0] ? 4'b0000
            : py[1] ? 4'b0001
            : py[2] ? 4'b0010
 	    : py[3] ? 4'b0011
            : py[4] ? 4'b0100
            : py[8] ? 4'b1000
            : py[9] ? 4'b1001
            : py[10] ? 4'b1010
            : py[11] ? 4'b1011
            : py[12] ? 4'b1100 
            : 4'b1111;

/******************* INTERRUPT ******************/

 
endmodule
   
